/**
 * A generic menu, intended to be used for both toggling options and invoking commands.
 *
 * Usage:
 *
 * var items = [
 *   {key: 'a', label: 'Item A', checked: true},
 *   {key: 'b', label: 'Item B'},
 *   '-',
 *   {key: 'random', label: 'Pick Random'}
 * ];
 * <Menu header="Header" items={items} onSelect={selectHandler} />
 *
 * 
 */

'use strict';function _typeof(obj) {"@babel/helpers - typeof";if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {_typeof = function _typeof(obj) {return typeof obj;};} else {_typeof = function _typeof(obj) {return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;};}return _typeof(obj);}


var _reactColor = require("react-color");
var _react = _interopRequireDefault(require("react"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError("Cannot call a class as a function");}}function _defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ("value" in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}function _createClass(Constructor, protoProps, staticProps) {if (protoProps) _defineProperties(Constructor.prototype, protoProps);if (staticProps) _defineProperties(Constructor, staticProps);return Constructor;}function _inherits(subClass, superClass) {if (typeof superClass !== "function" && superClass !== null) {throw new TypeError("Super expression must either be null or a function");}subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } });if (superClass) _setPrototypeOf(subClass, superClass);}function _setPrototypeOf(o, p) {_setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {o.__proto__ = p;return o;};return _setPrototypeOf(o, p);}function _createSuper(Derived) {var hasNativeReflectConstruct = _isNativeReflectConstruct();return function _createSuperInternal() {var Super = _getPrototypeOf(Derived),result;if (hasNativeReflectConstruct) {var NewTarget = _getPrototypeOf(this).constructor;result = Reflect.construct(Super, arguments, NewTarget);} else {result = Super.apply(this, arguments);}return _possibleConstructorReturn(this, result);};}function _possibleConstructorReturn(self, call) {if (call && (_typeof(call) === "object" || typeof call === "function")) {return call;}return _assertThisInitialized(self);}function _assertThisInitialized(self) {if (self === void 0) {throw new ReferenceError("this hasn't been initialised - super() hasn't been called");}return self;}function _isNativeReflectConstruct() {if (typeof Reflect === "undefined" || !Reflect.construct) return false;if (Reflect.construct.sham) return false;if (typeof Proxy === "function") return true;try {Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {}));return true;} catch (e) {return false;}}function _getPrototypeOf(o) {_getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {return o.__proto__ || Object.getPrototypeOf(o);};return _getPrototypeOf(o);}var
































Menu = /*#__PURE__*/function (_React$Component) {_inherits(Menu, _React$Component);var _super = _createSuper(Menu);



  function Menu(props) {var _this;_classCallCheck(this, Menu);
    _this = _super.call(this, props);
    _this.state = {
      showPalette: new Array(props.items.length).fill(false) };return _this;

  }

  // toggle color picker for menu items that have color enabled
  _createClass(Menu, [{ key: "toggleColorPicker", value: function toggleColorPicker(key, e) {
      if (this.state.showPalette[key] == false) {
        this.state.showPalette[key] = true;
        this.setState({ showPalette: this.state.showPalette });
      } else {
        this.state.showPalette[key] = false;
        this.setState({ showPalette: this.state.showPalette });
      }
    } }, { key: "clickHandler", value:

    function clickHandler(idx, e) {var togglePicker = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
      // do not call preventDefault on nullified events to avoid warnings
      if (e.eventPhase != null) {
        e.preventDefault();
      }
      var item = this.props.items[idx];
      if (typeof item == 'string') return; // for flow

      // propogate root update if new opts do not == old opts
      this.props.onClick(item);

      if (item.color && togglePicker) {
        this.toggleColorPicker(idx, e);
      }
    } }, { key: "handleColorChange", value:

    function handleColorChange(idx, color, e) {
      // update both hex and rgb values
      if (typeof this.props.items[idx] == 'string') return; // for flow not working
      if (typeof this.props.items[idx] === 'undefined') return; // for flow not working
      this.props.items[idx].color = {
        'hex': color.hex,
        'rgb': color.rgb };

      this.clickHandler(idx, e, false);
    } }, { key: "render", value:

    function render() {var _this2 = this;
      var makeHandler = function makeHandler(i) {return _this2.clickHandler.bind(_this2, i);};
      var makeColorPickerHandler = function makeColorPickerHandler(i) {return _this2.handleColorChange.bind(_this2, i);};

      var els = [];
      if (this.props.header) {
        els.push( /*#__PURE__*/_react["default"].createElement("div", { key: "header", className: "menu-header" }, this.props.header));
      }
      els = els.concat(this.props.items.map(function (item, i) {
        if (typeof item === 'string') {// would prefer "== '-'", see flow#1066
          return /*#__PURE__*/_react["default"].createElement("div", { key: i, className: "menu-separator" });
        } else {
          var checkClass = 'check' + (item.checked ? ' checked' : '');
          // initially hide color picker
          if (typeof item != 'string' && item.color) {
            var colorPicker = null;

            if (_this2.state.showPalette[i]) {
              colorPicker = /*#__PURE__*/
              _react["default"].createElement(_reactColor.SketchPicker, {
                color: item.color.rgb,
                onChangeComplete: makeColorPickerHandler(i) });

            }

            return /*#__PURE__*/(
              _react["default"].createElement("div", { key: i, className: "menu-item" }, /*#__PURE__*/
              _react["default"].createElement("span", { className: checkClass }), /*#__PURE__*/
              _react["default"].createElement("span", { key: i, onClick: makeHandler(i), className: "menu-item-label" }, item.label),
              colorPicker));


          } else {
            return /*#__PURE__*/(
              _react["default"].createElement("div", { key: i, className: "menu-item", onClick: makeHandler(i) }, /*#__PURE__*/
              _react["default"].createElement("span", { className: checkClass }), /*#__PURE__*/
              _react["default"].createElement("span", { className: "menu-item-label" }, item.label)));


          }
        }
      }));

      return /*#__PURE__*/(
        _react["default"].createElement("div", { className: "menu" },
        els));


    } }]);return Menu;}(_react["default"].Component);


module.exports = Menu;