/**
 * A store for sequences.
 *
 * This is used to store and retrieve reference data.
 *
 * 
 */
'use strict';




var _underscore = _interopRequireDefault(require("underscore"));

var _utils = require("./utils");function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}function _createForOfIteratorHelper(o, allowArrayLike) {var it;if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {if (it) o = it;var i = 0;var F = function F() {};return { s: F, n: function n() {if (i >= o.length) return { done: true };return { done: false, value: o[i++] };}, e: function e(_e) {throw _e;}, f: F };}throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");}var normalCompletion = true,didErr = false,err;return { s: function s() {it = o[Symbol.iterator]();}, n: function n() {var step = it.next();normalCompletion = step.done;return step;}, e: function e(_e2) {didErr = true;err = _e2;}, f: function f() {try {if (!normalCompletion && it["return"] != null) it["return"]();} finally {if (didErr) throw err;}} };}function _unsupportedIterableToArray(o, minLen) {if (!o) return;if (typeof o === "string") return _arrayLikeToArray(o, minLen);var n = Object.prototype.toString.call(o).slice(8, -1);if (n === "Object" && o.constructor) n = o.constructor.name;if (n === "Map" || n === "Set") return Array.from(o);if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);}function _arrayLikeToArray(arr, len) {if (len == null || len > arr.length) len = arr.length;for (var i = 0, arr2 = new Array(len); i < len; i++) {arr2[i] = arr[i];}return arr2;}function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError("Cannot call a class as a function");}}function _defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ("value" in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}function _createClass(Constructor, protoProps, staticProps) {if (protoProps) _defineProperties(Constructor.prototype, protoProps);if (staticProps) _defineProperties(Constructor, staticProps);return Constructor;}

// Store sequences of this many base pairs together in a single string.
var CHUNK_SIZE = 1000;








// This class stores sequences efficiently.
var SequenceStore = /*#__PURE__*/function () {
  // contig --> start of chunk --> sequence of chunk


  function SequenceStore() {_classCallCheck(this, SequenceStore);
    this.contigMap = {};
  }

  /**
   * Set a range of the genome to the particular sequence.
   * This overwrites any existing data.
   */_createClass(SequenceStore, [{ key: "setRange", value:
    function setRange(range, sequence) {
      if (range.length() === 0) return;
      if (range.length() != sequence.length) {
        throw 'setRange length mismatch';
      }

      var seqs = this._getSequences(range.contig);
      if (!seqs) {
        seqs = this.contigMap[range.contig] = {};
      }var _iterator = _createForOfIteratorHelper(

      this._chunkForInterval(range.interval)),_step;try {for (_iterator.s(); !(_step = _iterator.n()).done;) {var chunk = _step.value;
          var pos = chunk.chunkStart + chunk.offset - range.start();
          this._setChunk(seqs, chunk, sequence.slice(pos, pos + chunk.length));
        }} catch (err) {_iterator.e(err);} finally {_iterator.f();}
    }

    /**
     * Retrieve a range of sequence data.
     * If any portions are unknown, they will be set to '.'.
     */ }, { key: "getAsString", value:
    function getAsString(range) {
      var seqs = this._getSequences(range.contig);
      if (!seqs) {
        return '.'.repeat(range.length());
      }

      var chunks = this._chunkForInterval(range.interval);
      var result = '';var _iterator2 = _createForOfIteratorHelper(
      chunks),_step2;try {for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {var chunk = _step2.value;
          var seq = seqs[chunk.chunkStart];
          if (!seq) {
            result += '.'.repeat(chunk.length);
          } else if (chunk.offset === 0 && chunk.length == seq.length) {
            result += seq;
          } else {
            result += seq.slice(chunk.offset, chunk.offset + chunk.length);
          }
        }} catch (err) {_iterator2.e(err);} finally {_iterator2.f();}
      return result;
    }

    /**
     * Like getAsString(), but returns a less efficient object representation:
     * {'chr1:10': 'A', 'chr1:11': 'C', ...}
     */ }, { key: "getAsObjects", value:
    function getAsObjects(range) {
      return _underscore["default"].object(_underscore["default"].map(this.getAsString(range),
      function (base, i) {return [range.contig + ':' + (range.start() + i),
        base == '.' ? null : base];}));
    }

    // Retrieve a chunk from the sequence map.
  }, { key: "_getChunk", value: function _getChunk(seqs, start) {
      return seqs[start] || '.'.repeat(CHUNK_SIZE);
    }

    // Split an interval into chunks which align with the store.
  }, { key: "_chunkForInterval", value: function _chunkForInterval(range) {
      var offset = range.start % CHUNK_SIZE,
      chunkStart = range.start - offset;
      var chunks = [{
        chunkStart: chunkStart,
        offset: offset,
        length: Math.min(CHUNK_SIZE - offset, range.length()) }];

      chunkStart += CHUNK_SIZE;
      for (; chunkStart <= range.stop; chunkStart += CHUNK_SIZE) {
        chunks.push({
          chunkStart: chunkStart,
          offset: 0,
          length: Math.min(CHUNK_SIZE, range.stop - chunkStart + 1) });

      }
      return chunks;
    }

    // Set a (subset of a) chunk to the given sequence.
  }, { key: "_setChunk", value: function _setChunk(seqs, chunk, sequence) {
      // First: the easy case. Total replacement.
      if (chunk.offset === 0 && sequence.length == CHUNK_SIZE) {
        seqs[chunk.chunkStart] = sequence;
        return;
      }

      // We need to merge the new sequence with the old.
      var oldChunk = this._getChunk(seqs, chunk.chunkStart);
      seqs[chunk.chunkStart] = oldChunk.slice(0, chunk.offset) +
      sequence +
      oldChunk.slice(chunk.offset + sequence.length);
    }

    // Get the sequences for a contig, allowing chr- mismatches.
  }, { key: "_getSequences", value: function _getSequences(contig) {
      return this.contigMap[contig] ||
      this.contigMap[(0, _utils.altContigName)(contig)] ||
      null;
    } }]);return SequenceStore;}();


module.exports = SequenceStore;