/**
 * Fetcher/parser for gzipped Cytoband files. Cytoband files can be downloaded
 * or accessed from http://hgdownload.cse.ucsc.edu/goldenpath for a genome build.
 *
 * extracts CONTIG, START, END, NAME and VALUE
 *
 * 
 */
'use strict';



var _underscore = _interopRequireDefault(require("underscore"));
var _ContigInterval = _interopRequireDefault(require("../ContigInterval"));
var _chromosome = _interopRequireDefault(require("./chromosome"));
var _inflate = _interopRequireDefault(require("pako/lib/inflate"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError("Cannot call a class as a function");}}function _defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ("value" in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}function _createClass(Constructor, protoProps, staticProps) {if (protoProps) _defineProperties(Constructor.prototype, protoProps);if (staticProps) _defineProperties(Constructor, staticProps);return Constructor;} // for gzip inflation

function extractLine(cytoBandLine) {
  var split = cytoBandLine.split('\t');
  return {
    contig: split[0],
    band: {
      start: Number(split[1]),
      end: Number(split[2]),
      name: split[3],
      value: split[4] } };


}

function groupedBandsToChromosome(grouped) {
  var bands = _underscore["default"].map(grouped, function (g) {return g.band;});
  return new _chromosome["default"](
  { name: grouped[0].contig,
    bands: bands,
    position: new _ContigInterval["default"](grouped[0].contig, bands[0].start, bands.slice(-1)[0].end) });

}var

ImmediateCytoBandFile = /*#__PURE__*/function () {


  function ImmediateCytoBandFile(chrs) {_classCallCheck(this, ImmediateCytoBandFile);
    this.chrs = chrs;
  }_createClass(ImmediateCytoBandFile, [{ key: "getFeaturesInRange", value:

    function getFeaturesInRange(range) {
      return this.chrs[range.contig];
    } }]);return ImmediateCytoBandFile;}();var


CytoBandFile = /*#__PURE__*/function () {



  function CytoBandFile(remoteFile) {_classCallCheck(this, CytoBandFile);
    this.remoteFile = remoteFile;

    this.immediate = this.remoteFile.getAll().then(function (bytes) {
      var txt = _inflate["default"].inflate(bytes, { to: 'string' });
      var txtLines = _underscore["default"].filter(txt.split('\n'), function (i) {return i;}); // gets rid of empty lines
      var lines = txtLines.map(extractLine);
      return lines;
    }).then(function (lines) {
      // group bands by contig
      var grouped = _underscore["default"].groupBy(lines, function (l) {return l.contig;});
      var chrs = _underscore["default"].mapObject(grouped, function (g) {return groupedBandsToChromosome(g);});
      return new ImmediateCytoBandFile(chrs);
    });
    this.immediate.done();
  }_createClass(CytoBandFile, [{ key: "getFeaturesInRange", value:

    function getFeaturesInRange(range) {
      return this.immediate.then(function (immediate) {
        return immediate.getFeaturesInRange(range);
      });
    } }]);return CytoBandFile;}();


module.exports = {
  CytoBandFile: CytoBandFile };