/**
 * A data source which implements generic JSON protocol.
 * Currently only used to load alignments.
 * 
 */
'use strict';


var _feature = _interopRequireDefault(require("../data/feature"));

var _underscore = _interopRequireDefault(require("underscore"));
var _backbone = require("backbone");

var _ContigInterval = _interopRequireDefault(require("../ContigInterval"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}


function create(json) {

  // parse json
  var parsedJson = JSON.parse(json);
  var features = [];

  // fill features with json
  if (!_underscore["default"].isEmpty(parsedJson)) {
    features = _underscore["default"].values(parsedJson.features).map(function (feature) {return _feature["default"].fromGA4GH(feature);});
  }

  function rangeChanged(newRange) {
    // Data is already parsed, so immediately return
    var range = new _ContigInterval["default"](newRange.contig, newRange.start, newRange.stop);
    o.trigger('newdata', range);
    o.trigger('networkdone');
    return;
  }

  function getFeaturesInRange(range) {
    if (!range) return [];
    var r = _underscore["default"].filter(features, function (feature) {return feature.intersects(range);});
    return r;
  }

  var o = {
    rangeChanged: rangeChanged,
    getFeaturesInRange: getFeaturesInRange,

    on: function on() {},
    once: function once() {},
    off: function off() {},
    trigger: function trigger(string, any) {} };

  _underscore["default"].extend(o, _backbone.Events);
  return o;
}

module.exports = {
  create: create };