/**
 * A data source which implements generic JSON protocol.
 * Currently only used to load alignments.
 * 
 */
'use strict';


var _variant = require("../data/variant");

var _underscore = _interopRequireDefault(require("underscore"));
var _backbone = require("backbone");
var _q = _interopRequireDefault(require("q"));
var _ContigInterval = _interopRequireDefault(require("../ContigInterval"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}


function create(json) {

  // parse json
  var parsedJson = JSON.parse(json);
  var variants = [];
  var callSetNames = [];

  // fill variants with json
  if (!_underscore["default"].isEmpty(parsedJson)) {
    variants = _underscore["default"].values(parsedJson.variants).map(function (variant) {return new _variant.VariantContext(_variant.Variant.fromGA4GH(variant), variant.calls);});
    if (variants.length > 0) {// only call variants if non-empty
      callSetNames = _underscore["default"].map(variants[0].calls, function (c) {return c.callSetName;});
    }
  }

  function rangeChanged(newRange) {
    // Data is already parsed, so immediately return
    var range = new _ContigInterval["default"](newRange.contig, newRange.start, newRange.stop);
    o.trigger('newdata', range);
    o.trigger('networkdone');
    return;
  }

  function getVariantsInRange(range) {
    if (!range) return [];
    var filtered = _underscore["default"].filter(variants, function (variant) {return variant.intersects(range);});
    return _underscore["default"].map(filtered, function (f) {return f.variant;});
  }

  function getGenotypesInRange(range) {
    if (!range) return [];

    return _underscore["default"].filter(variants, function (variant) {return variant.intersects(range);});
  }

  function getCallNames() {
    return _q["default"].resolve(callSetNames);
  }

  var o = {
    rangeChanged: rangeChanged,
    getVariantsInRange: getVariantsInRange,
    getGenotypesInRange: getGenotypesInRange,
    getCallNames: getCallNames,

    on: function on() {},
    once: function once() {},
    off: function off() {},
    trigger: function trigger(string, any) {} };

  _underscore["default"].extend(o, _backbone.Events);
  return o;
}

module.exports = {
  create: create };