/**
 * A data source which implements generic JSON protocol.
 * Currently only used to load alignments.
 * 
 */
'use strict';


var _chromosome = _interopRequireDefault(require("../data/chromosome"));

var _underscore = _interopRequireDefault(require("underscore"));
var _backbone = require("backbone");

var _ContigInterval = _interopRequireDefault(require("../ContigInterval"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}


function create(json) {

  // parse json
  var parsedJson = JSON.parse(json);
  var chromosomes = [];

  // fill chromosomes with json
  if (!_underscore["default"].isEmpty(parsedJson)) {
    chromosomes = _underscore["default"].values(parsedJson).map(function (chr) {return new _chromosome["default"](chr);});
  }

  function rangeChanged(newRange) {
    // Data is already parsed, so immediately return
    var range = new _ContigInterval["default"](newRange.contig, newRange.start, newRange.stop);
    o.trigger('newdata', range);
    o.trigger('networkdone');
    return;
  }

  function getFeaturesInRange(range) {
    return _underscore["default"].filter(chromosomes, function (chr) {return range.chrOnContig(chr.name);});
  }

  var o = {
    rangeChanged: rangeChanged,
    getFeaturesInRange: getFeaturesInRange,

    on: function on() {},
    once: function once() {},
    off: function off() {},
    trigger: function trigger(string, any) {} };

  _underscore["default"].extend(o, _backbone.Events);
  return o;
}

module.exports = {
  create: create };