
'use strict';

var _underscore = _interopRequireDefault(require("underscore"));
var _q = _interopRequireDefault(require("q"));
var _backbone = require("backbone");

var _ContigInterval = _interopRequireDefault(require("../ContigInterval"));
var _bam = _interopRequireDefault(require("../data/bam"));
var _RemoteFile = _interopRequireDefault(require("../RemoteFile"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}




// Genome ranges are rounded to multiples of this for fetching.
// This reduces network activity while fetching.
// TODO: tune this value
var BASE_PAIRS_PER_FETCH = 100;
var ZERO_BASED = false;


function createFromBamFile(remoteSource) {
  var reads = {};

  // Mapping from contig name to canonical contig name.
  var contigNames = {};

  // Ranges for which we have complete information -- no need to hit network.
  var coveredRanges = [];

  function addRead(read) {
    var key = read.getKey();
    if (!reads[key]) {
      reads[key] = read;
    }
  }

  function saveContigMapping(header) {
    header.references.forEach(function (ref) {
      var name = ref.name;
      contigNames[name] = name;
      contigNames['chr' + name] = name;
      if (name.slice(0, 3) == 'chr') {
        contigNames[name.slice(3)] = name;
      }
    });
  }

  function fetch(range) {
    var refsPromise = !_underscore["default"].isEmpty(contigNames) ? _q["default"].when() :
    remoteSource.header.then(saveContigMapping);

    // For BAMs without index chunks, we need to fetch the entire BAI file
    // before we can know how large the BAM header is. If the header is
    // pending, it's almost certainly because the BAI file is in flight.
    _q["default"].when().then(function () {
      if (refsPromise.isPending() && !remoteSource.hasIndexChunks) {
        o.trigger('networkprogress', {
          status: 'Fetching BAM index -- use index chunks to speed this up' });

      }
    }).done();

    return refsPromise.then(function () {
      var contigName = contigNames[range.contig];
      var interval = new _ContigInterval["default"](contigName, range.start, range.stop);

      // Check if this interval is already in the cache.
      // If not, immediately "cover" it to prevent duplicate requests.
      if (interval.isCoveredBy(coveredRanges)) {
        return _q["default"].when();
      }

      interval = interval.round(BASE_PAIRS_PER_FETCH, ZERO_BASED);
      var newRanges = interval.complementIntervals(coveredRanges);
      coveredRanges.push(interval);
      coveredRanges = _ContigInterval["default"].coalesce(coveredRanges);

      return _q["default"].all(newRanges.map(function (range) {return (
          remoteSource.getFeaturesInRange(range).
          progress(function (progressEvent) {
            o.trigger('networkprogress', progressEvent);
          }).
          then(function (reads) {
            reads.forEach(function (read) {return addRead(read);});
            o.trigger('networkdone');
            o.trigger('newdata', range);
          }));}));
    });
  }

  function getFeaturesInRange(range) {
    if (!range) return [];
    if (_underscore["default"].isEmpty(contigNames)) return [];

    var canonicalRange = new _ContigInterval["default"](contigNames[range.contig],
    range.start(), range.stop());

    return _underscore["default"].filter(reads, function (read) {return read.intersects(canonicalRange);});
  }

  var o = {
    rangeChanged: function rangeChanged(newRange) {
      fetch(newRange).done();
    },
    getFeaturesInRange: getFeaturesInRange,

    // These are here to make Flow happy.
    on: function on() {},
    once: function once() {},
    off: function off() {},
    trigger: function trigger(status, param) {} };

  _underscore["default"].extend(o, _backbone.Events); // Make this an event emitter

  return o;
}







function create(spec) {
  var url = spec.url;
  if (!url) {
    throw new Error("Missing URL from track data: ".concat(JSON.stringify(spec)));
  }
  var indexUrl = spec.indexUrl;
  if (!indexUrl) {
    throw new Error("Missing indexURL from track data: ".concat(JSON.stringify(spec)));
  }

  // TODO: this is overly repetitive, see flow issue facebook/flow#437
  var bamFile = spec.indexChunks ?
  new _bam["default"](new _RemoteFile["default"](url), new _RemoteFile["default"](indexUrl), spec.indexChunks) :
  new _bam["default"](new _RemoteFile["default"](url), new _RemoteFile["default"](indexUrl));
  return createFromBamFile(bamFile);
}

module.exports = {
  create: create,
  createFromBamFile: createFromBamFile };