
'use strict';


var _Alignment = require("../Alignment");

var _underscore = _interopRequireDefault(require("underscore"));
var _q = _interopRequireDefault(require("q"));
var _backbone = require("backbone");

var _ContigInterval = _interopRequireDefault(require("../ContigInterval"));
var _Interval = _interopRequireDefault(require("../Interval"));
var _BigBed = _interopRequireDefault(require("../data/BigBed"));
var _gene = _interopRequireDefault(require("../data/gene"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}function _slicedToArray(arr, i) {return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();}function _nonIterableRest() {throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");}function _unsupportedIterableToArray(o, minLen) {if (!o) return;if (typeof o === "string") return _arrayLikeToArray(o, minLen);var n = Object.prototype.toString.call(o).slice(8, -1);if (n === "Object" && o.constructor) n = o.constructor.name;if (n === "Map" || n === "Set") return Array.from(o);if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);}function _arrayLikeToArray(arr, len) {if (len == null || len > arr.length) len = arr.length;for (var i = 0, arr2 = new Array(len); i < len; i++) {arr2[i] = arr[i];}return arr2;}function _iterableToArrayLimit(arr, i) {if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return;var _arr = [];var _n = true;var _d = false;var _e = undefined;try {for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {_arr.push(_s.value);if (i && _arr.length === i) break;}} catch (err) {_d = true;_e = err;} finally {try {if (!_n && _i["return"] != null) _i["return"]();} finally {if (_d) throw _e;}}return _arr;}function _arrayWithHoles(arr) {if (Array.isArray(arr)) return arr;}



// The fields are described at http://genome.ucsc.edu/FAQ/FAQformat#format1
// Fields 4-12 are optional
function parseBedFeature(f) {
  var position = new _ContigInterval["default"](f.contig, f.start, f.stop),
  x = f.rest.split('\t');

  // if no id, generate randomly for unique storage
  var id = x[0] ? x[0] : position.toString(); // e.g. ENST00000359597
  var score = x[1] ? parseInt(x[1]) : 1000; // number from 0-1000
  var strand = (0, _Alignment.strToStrand)(x[2]); // either +, - or .
  var codingRegion = x[3] && x[4] ? new _Interval["default"](Number(x[3]), Number(x[4])) : new _Interval["default"](f.start, f.stop);
  var geneId = x[9] ? x[9] : id;
  var name = x[10] ? x[10] : "";

  // parse exons
  var exons = [];
  if (x[7] && x[8]) {
    // exons arrays sometimes have trailing commas
    var exonLengths = x[7].replace(/,*$/, '').split(',').map(Number),
    exonStarts = x[8].replace(/,*$/, '').split(',').map(Number);

    exons = _underscore["default"].zip(exonStarts, exonLengths).
    map(function (_ref) {var _ref2 = _slicedToArray(_ref, 2),start = _ref2[0],length = _ref2[1];
      return new _Interval["default"](f.start + start, f.start + start + length);
    });
  }

  return new _gene["default"]({
    position: position,
    id: id,
    score: score,
    strand: strand,
    codingRegion: codingRegion,
    geneId: geneId,
    name: name,
    exons: exons });

}

function createFromBigBedFile(remoteSource) {
  // Collection of genes that have already been loaded.
  var genes = {};

  // Ranges for which we have complete information -- no need to hit network.
  var coveredRanges = [];

  function addGene(newGene) {
    if (!genes[newGene.id]) {
      genes[newGene.id] = newGene;
    }
  }

  function getFeaturesInRange(range) {
    if (!range) return [];
    var results = [];
    _underscore["default"].each(genes, function (gene) {
      if (range.intersects(gene.position)) {
        results.push(gene);
      }
    });
    return results;
  }

  function fetch(range) {
    var interval = new _ContigInterval["default"](range.contig, range.start, range.stop);

    // Check if this interval is already in the cache.
    if (interval.isCoveredBy(coveredRanges)) {
      return _q["default"].when();
    }

    coveredRanges.push(interval);
    coveredRanges = _ContigInterval["default"].coalesce(coveredRanges);

    return remoteSource.getFeatureBlocksOverlapping(interval).then(function (featureBlocks) {
      featureBlocks.forEach(function (fb) {
        coveredRanges.push(fb.range);
        coveredRanges = _ContigInterval["default"].coalesce(coveredRanges);
        var genes = fb.rows.map(parseBedFeature);
        genes.forEach(function (gene) {return addGene(gene);});
      });
      //we have new data from our internal block range
      o.trigger('newdata', interval);
      o.trigger('networkdone');
    });
  }

  var o = {
    rangeChanged: function rangeChanged(newRange) {
      fetch(newRange).done();
    },
    getFeaturesInRange: getFeaturesInRange,

    // These are here to make Flow happy.
    on: function on() {},
    once: function once() {},
    off: function off() {},
    trigger: function trigger(status, param) {} };

  _underscore["default"].extend(o, _backbone.Events); // Make this an event emitter

  return o;
}

function create(data) {
  var url = data.url;
  if (!url) {
    throw new Error("Missing URL from track: ".concat(JSON.stringify(data)));
  }

  return createFromBigBedFile(new _BigBed["default"](url));
}

module.exports = {
  create: create,
  createFromBigBedFile: createFromBigBedFile };