/**
 * The "glue" between cytoBand.js and IdiogramTrack.js.
 *
 * Allows loading remote gzipped cytoband files into an Idiogram visualization.
 *
 * 
 */
'use strict';

var _underscore = _interopRequireDefault(require("underscore"));
var _backbone = require("backbone");

var _ContigInterval = _interopRequireDefault(require("../ContigInterval"));
var _chromosome = _interopRequireDefault(require("../data/chromosome"));

var _cytoBand = require("../data/cytoBand");


var _RemoteFile = _interopRequireDefault(require("../RemoteFile"));
var _utils = _interopRequireDefault(require("../utils"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}

var createFromCytoBandFile = function createFromCytoBandFile(remoteSource) {
  // Local cache of genomic data.
  var contigMap = {};

  // This either adds or removes a 'chr' as needed.
  function normalizeRange(range) {
    if (contigMap[range.contig] !== undefined) {
      return range;
    }
    var altContig = _utils["default"].altContigName(range.contig);
    if (contigMap[altContig] !== undefined) {
      return new _ContigInterval["default"](altContig, range.start(), range.stop());
    }
    return range;
  }

  function fetch(range) {
    remoteSource.getFeaturesInRange(range).
    then(function (chr) {
      contigMap[chr.name] = chr;
    }).then(function () {
      o.trigger('newdata', range);
    }).done();
  }

  function getFeaturesInRange(range) {
    return [contigMap[normalizeRange(range).contig]];
  }

  var o = {
    // The range here is 0-based, inclusive
    rangeChanged: function rangeChanged(newRange) {
      // Check if this interval is already in the cache.
      if (contigMap[newRange.contig] !== undefined) {
        return;
      }
      fetch(new _ContigInterval["default"](newRange.contig, newRange.start, newRange.stop));
    },
    getFeaturesInRange: getFeaturesInRange,
    // These are here to make Flow happy.
    on: function on() {},
    once: function once() {},
    off: function off() {},
    trigger: function trigger(status, param) {} };

  _underscore["default"].extend(o, _backbone.Events); // Make this an event emitter

  return o;
};

function create(url) {
  if (!url) {
    throw new Error("Missing URL from track: ".concat(url));
  }
  return createFromCytoBandFile(new _cytoBand.CytoBandFile(new _RemoteFile["default"](url)));
}


module.exports = {
  create: create,
  createFromCytoBandFile: createFromCytoBandFile };