/**
 * Caching & prefetching for VCF sources.
 *
 * 
 */
'use strict';


var _backbone = _interopRequireDefault(require("backbone"));
var _underscore = _interopRequireDefault(require("underscore"));
var _q = _interopRequireDefault(require("q"));

var _ContigInterval = _interopRequireDefault(require("../ContigInterval"));
var _RemoteFile = _interopRequireDefault(require("../RemoteFile"));
var _LocalStringFile = _interopRequireDefault(require("../LocalStringFile"));


var _variant = require("../data/variant");
var _vcf = require("../data/vcf");function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };} // requirement for jshint to pass
/* exported Variant */












var BASE_PAIRS_PER_FETCH = 100;
var ZERO_BASED = false;

function variantContextKey(v) {
  return "".concat(v.variant.contig, ":").concat(v.variant.position);
}


function createFromVcfFile(remoteSource) {
  var variants = {};

  // Ranges for which we have complete information -- no need to hit network.
  var coveredRanges = [];

  function addVariantContext(v) {
    var key = variantContextKey(v);
    if (!variants[key]) {
      variants[key] = v;
    }
  }

  function fetch(range) {
    var interval = new _ContigInterval["default"](range.contig, range.start, range.stop);

    // Check if this interval is already in the cache.
    if (interval.isCoveredBy(coveredRanges)) {
      return _q["default"].when();
    }

    interval = interval.round(BASE_PAIRS_PER_FETCH, ZERO_BASED);

    // "Cover" the range immediately to prevent duplicate fetches.
    coveredRanges.push(interval);
    coveredRanges = _ContigInterval["default"].coalesce(coveredRanges);
    return remoteSource.getFeaturesInRange(interval).then(function (variants) {
      variants.forEach(function (variant) {return addVariantContext(variant);});
      o.trigger('newdata', interval);
    });
  }

  function getVariantsInRange(range) {
    if (!range) return []; // XXX why would this happen?
    var filtered = _underscore["default"].filter(variants, function (v) {return range.containsLocus(v.variant.contig, v.variant.position);});
    return _underscore["default"].map(filtered, function (f) {return f.variant;});
  }

  function getGenotypesInRange(range) {
    return _underscore["default"].filter(variants, function (v) {return range.containsLocus(v.variant.contig, v.variant.position);});
  }

  function getCallNames() {
    return remoteSource.getCallNames().then(function (samples) {
      return samples;
    });
  }

  var o = {
    rangeChanged: function rangeChanged(newRange) {
      fetch(newRange).done();
    },
    getVariantsInRange: getVariantsInRange,
    getGenotypesInRange: getGenotypesInRange,
    getCallNames: getCallNames,

    // These are here to make Flow happy.
    on: function on() {},
    off: function off() {},
    trigger: function trigger(status, param) {} };

  _underscore["default"].extend(o, _backbone["default"]); // Make this an event emitter

  return o;
}

function create(data) {var
  url = data.url,content = data.content;
  if (url) {
    return createFromVcfFile(new _vcf.VcfFile(new _RemoteFile["default"](url)));
  } else if (content) {
    return createFromVcfFile(new _vcf.VcfFile(new _LocalStringFile["default"](content)));
  }
  // If no URL or content is passed, fail
  throw new Error("Missing URL or content from track: ".concat(JSON.stringify(data)));
}

module.exports = {
  create: create,
  createFromVcfFile: createFromVcfFile };