/**
 * Data management for CoverageTrack.
 *
 * This class tracks counts and mismatches at each locus.
 *
 * 
 */
'use strict';


var _feature = _interopRequireDefault(require("../data/feature"));



var _utils = _interopRequireDefault(require("../utils"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}function _createForOfIteratorHelper(o, allowArrayLike) {var it;if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {if (it) o = it;var i = 0;var F = function F() {};return { s: F, n: function n() {if (i >= o.length) return { done: true };return { done: false, value: o[i++] };}, e: function e(_e) {throw _e;}, f: F };}throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");}var normalCompletion = true,didErr = false,err;return { s: function s() {it = o[Symbol.iterator]();}, n: function n() {var step = it.next();normalCompletion = step.done;return step;}, e: function e(_e2) {didErr = true;err = _e2;}, f: function f() {try {if (!normalCompletion && it["return"] != null) it["return"]();} finally {if (didErr) throw err;}} };}function _unsupportedIterableToArray(o, minLen) {if (!o) return;if (typeof o === "string") return _arrayLikeToArray(o, minLen);var n = Object.prototype.toString.call(o).slice(8, -1);if (n === "Object" && o.constructor) n = o.constructor.name;if (n === "Map" || n === "Set") return Array.from(o);if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);}function _arrayLikeToArray(arr, len) {if (len == null || len > arr.length) len = arr.length;for (var i = 0, arr2 = new Array(len); i < len; i++) {arr2[i] = arr[i];}return arr2;}function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError("Cannot call a class as a function");}}function _defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ("value" in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}function _createClass(Constructor, protoProps, staticProps) {if (protoProps) _defineProperties(Constructor.prototype, protoProps);if (staticProps) _defineProperties(Constructor, staticProps);return Constructor;}








// This class provides data management for the visualization, grouping paired
// reads and managing the pileup.
var CoverageCache = /*#__PURE__*/function () {
  // maps groupKey to VisualGroup

  // ref --> position --> BinSummary



  function CoverageCache(referenceSource) {_classCallCheck(this, CoverageCache);
    this.items = {};
    this.refToCounts = {};
    this.referenceSource = referenceSource;
  }

  // Load a new read into the visualization cache.
  // Calling this multiple times with the same read is a no-op.
  _createClass(CoverageCache, [{ key: "addItem", value: function addItem(item) {
      var key = item.getKey();
      if (key in this.items) return; // we've already seen this one.
      this.items[key] = item;
      var coverageCount = item.getCoverage(this.referenceSource);
      this.addToCoverage(coverageCount);
    }

    // Updates reference mismatch information for previously-loaded reads.
  }, { key: "updateMismatches", value: function updateMismatches(range) {
      var ref = this._canonicalRef(range.contig);
      this.refToCounts[ref] = {}; // TODO: could be more efficient

      for (var k in this.items) {
        var item = this.items[k];
        if (item.getInterval().chrOnContig(range.contig)) {
          var coverageCount = item.getCoverage(this.referenceSource);
          this.addToCoverage(coverageCount);
        }
      }
    } }, { key: "addToCoverage", value:

    function addToCoverage(coverageCount) {

      // Add coverage/mismatch information
      var ref = this._canonicalRef(coverageCount.range.contig);
      if (!(ref in this.refToCounts)) {
        this.refToCounts[ref] = {};
      }
      var counts = this.refToCounts[ref],
      contig = coverageCount.range,
      start = contig.start(),
      stop = contig.stop();
      for (var pos = start; pos <= stop; pos++) {
        var c = counts[pos];
        if (!c) {
          counts[pos] = c = { count: 0 };
        }
        c.count += 1;
      }

      if (coverageCount.opInfo) {var _iterator = _createForOfIteratorHelper(
        coverageCount.opInfo.mismatches),_step;try {for (_iterator.s(); !(_step = _iterator.n()).done;) {var mm = _step.value;
            var bin = counts[mm.pos];
            var mismatches;
            if (bin.mismatches) {
              mismatches = bin.mismatches;
            } else {
              mismatches = bin.mismatches = {};
              bin.ref = this.referenceSource.getRangeAsString({
                contig: ref, start: mm.pos, stop: mm.pos });
            }
            var _c = mismatches[mm.basePair] || 0;
            mismatches[mm.basePair] = 1 + _c;
          }} catch (err) {_iterator.e(err);} finally {_iterator.f();}
      }

    } }, { key: "maxCoverageForRange", value:

    function maxCoverageForRange(range) {
      var bins = this.refToCounts[range.contig] ||
      this.refToCounts[_utils["default"].altContigName(range.contig)] ||
      {};

      var max = 0; // minimum coverage
      for (var i = range.start(); i < range.stop(); i++) {
        if (bins[i] != undefined) {
          if (bins[i].count > max) {
            max = bins[i].count;
          }
        }
      }
      return max;
    } }, { key: "binsForRef", value:

    function binsForRef(ref) {
      return this.refToCounts[ref] ||
      this.refToCounts[_utils["default"].altContigName(ref)] ||
      {};
    }

    // Returns whichever form of the ref ("chr17", "17") has been seen.
  }, { key: "_canonicalRef", value: function _canonicalRef(ref) {
      if (this.refToCounts[ref]) return ref;
      var alt = _utils["default"].altContigName(ref);
      if (this.refToCounts[alt]) return alt;
      return ref;
    } }]);return CoverageCache;}();


module.exports = CoverageCache;