/**
 * Visualization of genes, including exons and coding regions.
 * 
 */
'use strict';function _typeof(obj) {"@babel/helpers - typeof";if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {_typeof = function _typeof(obj) {return typeof obj;};} else {_typeof = function _typeof(obj) {return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;};}return _typeof(obj);}


var _gene = _interopRequireDefault(require("../data/gene"));




var _genericFeature = _interopRequireDefault(require("../data/genericFeature"));
var _GenericFeatureCache = require("./GenericFeatureCache");

var _react = _interopRequireDefault(require("react"));
var _underscore = _interopRequireDefault(require("underscore"));
var _shallowEquals = _interopRequireDefault(require("shallow-equals"));

var _bedtools = _interopRequireDefault(require("../data/bedtools"));
var _Interval = _interopRequireDefault(require("../Interval"));
var _d3utils = _interopRequireDefault(require("./d3utils"));
var _scale = _interopRequireDefault(require("../scale"));
var _ContigInterval = _interopRequireDefault(require("../ContigInterval"));
var _canvasUtils = _interopRequireDefault(require("./canvas-utils"));
var _utils = _interopRequireDefault(require("../utils"));
var _dataCanvas = _interopRequireDefault(require("data-canvas"));
var _style = _interopRequireDefault(require("../style"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError("Cannot call a class as a function");}}function _defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ("value" in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}function _createClass(Constructor, protoProps, staticProps) {if (protoProps) _defineProperties(Constructor.prototype, protoProps);if (staticProps) _defineProperties(Constructor, staticProps);return Constructor;}function _inherits(subClass, superClass) {if (typeof superClass !== "function" && superClass !== null) {throw new TypeError("Super expression must either be null or a function");}subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } });if (superClass) _setPrototypeOf(subClass, superClass);}function _setPrototypeOf(o, p) {_setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {o.__proto__ = p;return o;};return _setPrototypeOf(o, p);}function _createSuper(Derived) {var hasNativeReflectConstruct = _isNativeReflectConstruct();return function _createSuperInternal() {var Super = _getPrototypeOf(Derived),result;if (hasNativeReflectConstruct) {var NewTarget = _getPrototypeOf(this).constructor;result = Reflect.construct(Super, arguments, NewTarget);} else {result = Super.apply(this, arguments);}return _possibleConstructorReturn(this, result);};}function _possibleConstructorReturn(self, call) {if (call && (_typeof(call) === "object" || typeof call === "function")) {return call;}return _assertThisInitialized(self);}function _assertThisInitialized(self) {if (self === void 0) {throw new ReferenceError("this hasn't been initialised - super() hasn't been called");}return self;}function _isNativeReflectConstruct() {if (typeof Reflect === "undefined" || !Reflect.construct) return false;if (Reflect.construct.sham) return false;if (typeof Proxy === "function") return true;try {Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {}));return true;} catch (e) {return false;}}function _getPrototypeOf(o) {_getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {return o.__proto__ || Object.getPrototypeOf(o);};return _getPrototypeOf(o);}


// Draw an arrow in the middle of the visible portion of range.
function drawArrow(ctx,
clampedScale,
range,
tipY,
strand) {
  var x1 = clampedScale(1 + range.start),
  x2 = clampedScale(1 + range.stop);

  // it's off-screen or there's not enough room to draw it legibly.
  if (x2 - x1 <= 2 * _style["default"].GENE_ARROW_SIZE) return;

  var cx = (x1 + x2) / 2;
  ctx.beginPath();
  if (strand == '-') {
    ctx.moveTo(cx + _style["default"].GENE_ARROW_SIZE, tipY - _style["default"].GENE_ARROW_SIZE);
    ctx.lineTo(cx, tipY);
    ctx.lineTo(cx + _style["default"].GENE_ARROW_SIZE, tipY + _style["default"].GENE_ARROW_SIZE);
  } else {
    ctx.moveTo(cx - _style["default"].GENE_ARROW_SIZE, tipY - _style["default"].GENE_ARROW_SIZE);
    ctx.lineTo(cx, tipY);
    ctx.lineTo(cx - _style["default"].GENE_ARROW_SIZE, tipY + _style["default"].GENE_ARROW_SIZE);
  }
  ctx.stroke();
}

function drawGeneName(ctx,
clampedScale,
geneLineY,
gene,
textIntervals) {
  var p = gene.position,
  centerX = 0.5 * (clampedScale(1 + p.start()) + clampedScale(1 + p.stop()));
  // do not use gene name if it is null or empty
  var name = !_underscore["default"].isEmpty(_utils["default"].stringToLiteral(gene.name)) ? gene.name : gene.id;
  var textWidth = ctx.measureText(name).width;
  var textInterval = new _Interval["default"](centerX - 0.5 * textWidth,
  centerX + 0.5 * textWidth);
  if (!_underscore["default"].any(textIntervals, function (iv) {return textInterval.intersects(iv);})) {
    textIntervals.push(textInterval);
    var baselineY = geneLineY + _style["default"].GENE_FONT_SIZE + _style["default"].GENE_TEXT_PADDING;
    ctx.fillText(name, centerX, baselineY);
  }
}var

GeneTrack = /*#__PURE__*/function (_React$Component) {_inherits(GeneTrack, _React$Component);var _super = _createSuper(GeneTrack);





  function GeneTrack(props) {var _this;_classCallCheck(this, GeneTrack);
    _this = _super.call(this, props);
    _this.ref = /*#__PURE__*/_react["default"].createRef();
    _this.state = {
      networkStatus: null };return _this;

  }_createClass(GeneTrack, [{ key: "render", value:

    function render() {
      return /*#__PURE__*/_react["default"].createElement("canvas", { ref: this.ref });
    } }, { key: "componentDidMount", value:

    function componentDidMount() {var _this2 = this;
      this.cache = new _GenericFeatureCache.GenericFeatureCache(this.props.referenceSource);
      // Visualize new reference data as it comes in from the network.
      this.props.source.on('newdata', function (range) {
        // add genes to generic cache
        var genes = _this2.props.source.getFeaturesInRange(range);
        genes.forEach(function (f) {return _this2.cache.addFeature(new _genericFeature["default"](f.id, f.position, f));});
        _this2.updateVisualization();
      });
      this.props.referenceSource.on('newdata', function (range) {
        _this2.updateVisualization();
      });
      this.props.source.on('networkprogress', function (e) {
        _this2.setState({ networkStatus: e });
      });
      this.props.source.on('networkdone', function (e) {
        _this2.setState({ networkStatus: null });
      });

      this.updateVisualization();
    } }, { key: "getScale", value:

    function getScale() {
      return _d3utils["default"].getTrackScale(this.props.range, this.props.width);
    } }, { key: "componentDidUpdate", value:

    function componentDidUpdate(prevProps, prevState) {
      if (!(0, _shallowEquals["default"])(this.props, prevProps) ||
      !(0, _shallowEquals["default"])(this.state, prevState)) {
        this.updateVisualization();
      }
    } }, { key: "updateVisualization", value:

    function updateVisualization() {
      var canvas = this.ref.current;var _this$props =
      this.props,width = _this$props.width,height = _this$props.height,
      genomeRange = this.props.range;

      var range = new _ContigInterval["default"](genomeRange.contig, genomeRange.start, genomeRange.stop);

      // Hold off until height & width are known.
      if (width === 0) return;

      var sc = this.getScale(),
      // We can't clamp scale directly because of offsetPx.
      clampedScale = _scale["default"].linear().
      domain([sc.invert(0), sc.invert(width)]).
      range([0, width]).
      clamp(true);



      if (canvas && canvas instanceof Element) {// check for getContext
        if (canvas instanceof HTMLCanvasElement) {// check for sizeCanvas
          _d3utils["default"].sizeCanvas(canvas, width, height);
        }
        var ctx = _dataCanvas["default"].getDataContext(_canvasUtils["default"].getContext(canvas));

        ctx.reset();
        ctx.clearRect(0, 0, ctx.canvas.width, ctx.canvas.height);

        var geneLineY = Math.round(height / 4);
        var textIntervals = []; // x-intervals with rendered gene names, to avoid over-drawing.
        ctx.font = "".concat(_style["default"].GENE_FONT_SIZE, "px ").concat(_style["default"].GENE_FONT);
        ctx.textAlign = 'center';
        var vFeatures = _underscore["default"].flatten(_underscore["default"].map(this.cache.getGroupsOverlapping(range),
        function (g) {return g.items;}));

        vFeatures.forEach(function (vFeature) {
          var gene = vFeature.gFeature;
          if (!gene.position.intersects(range)) return;
          ctx.pushObject(gene);
          ctx.lineWidth = 1;
          ctx.strokeStyle = _style["default"].GENE_COLOR;
          ctx.fillStyle = _style["default"].GENE_COLOR;

          _canvasUtils["default"].drawLine(ctx, clampedScale(1 + gene.position.start()), geneLineY + 0.5,
          clampedScale(1 + gene.position.stop()), geneLineY + 0.5);

          // TODO: only compute all these intervals when data becomes available.
          var exons = _bedtools["default"].splitCodingExons(gene.exons, gene.codingRegion);
          exons.forEach(function (exon) {
            ctx.fillRect(sc(1 + exon.start),
            geneLineY - 3 * (exon.isCoding ? 2 : 1),
            sc(exon.stop + 2) - sc(1 + exon.start),
            6 * (exon.isCoding ? 2 : 1));
          });

          var introns = gene.position.interval.complementIntervals(gene.exons);
          introns.forEach(function (range) {
            drawArrow(ctx, clampedScale, range, geneLineY + 0.5, gene.strand);
          });
          ctx.strokeStyle = _style["default"].GENE_COMPLEMENT_COLOR;
          ctx.lineWidth = 2;
          gene.exons.forEach(function (range) {
            drawArrow(ctx, clampedScale, range, geneLineY + 0.5, gene.strand);
          });

          drawGeneName(ctx, clampedScale, geneLineY, gene, textIntervals);

          ctx.popObject();
        });
      } // end typecheck for canvas
    } }]);return GeneTrack;}(_react["default"].Component);


GeneTrack.displayName = 'genes';

module.exports = GeneTrack;