/**
 * A track which displays a reference genome.
 * 
 */
'use strict';function _typeof(obj) {"@babel/helpers - typeof";if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {_typeof = function _typeof(obj) {return typeof obj;};} else {_typeof = function _typeof(obj) {return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;};}return _typeof(obj);}






var _react = _interopRequireDefault(require("react"));
var _shallowEquals = _interopRequireDefault(require("shallow-equals"));


var _canvasUtils = _interopRequireDefault(require("./canvas-utils"));
var _ContigInterval = _interopRequireDefault(require("../ContigInterval"));
var _dataCanvas = _interopRequireDefault(require("data-canvas"));
var _d3utils = _interopRequireDefault(require("./d3utils"));
var _DisplayMode = _interopRequireDefault(require("./DisplayMode"));
var _TiledCanvas2 = _interopRequireDefault(require("./TiledCanvas"));
var _style = _interopRequireDefault(require("../style"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError("Cannot call a class as a function");}}function _defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ("value" in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}function _createClass(Constructor, protoProps, staticProps) {if (protoProps) _defineProperties(Constructor.prototype, protoProps);if (staticProps) _defineProperties(Constructor, staticProps);return Constructor;}function _inherits(subClass, superClass) {if (typeof superClass !== "function" && superClass !== null) {throw new TypeError("Super expression must either be null or a function");}subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } });if (superClass) _setPrototypeOf(subClass, superClass);}function _setPrototypeOf(o, p) {_setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {o.__proto__ = p;return o;};return _setPrototypeOf(o, p);}function _createSuper(Derived) {var hasNativeReflectConstruct = _isNativeReflectConstruct();return function _createSuperInternal() {var Super = _getPrototypeOf(Derived),result;if (hasNativeReflectConstruct) {var NewTarget = _getPrototypeOf(this).constructor;result = Reflect.construct(Super, arguments, NewTarget);} else {result = Super.apply(this, arguments);}return _possibleConstructorReturn(this, result);};}function _possibleConstructorReturn(self, call) {if (call && (_typeof(call) === "object" || typeof call === "function")) {return call;}return _assertThisInitialized(self);}function _assertThisInitialized(self) {if (self === void 0) {throw new ReferenceError("this hasn't been initialised - super() hasn't been called");}return self;}function _isNativeReflectConstruct() {if (typeof Reflect === "undefined" || !Reflect.construct) return false;if (Reflect.construct.sham) return false;if (typeof Proxy === "function") return true;try {Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {}));return true;} catch (e) {return false;}}function _getPrototypeOf(o) {_getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {return o.__proto__ || Object.getPrototypeOf(o);};return _getPrototypeOf(o);}


function renderGenome(ctx,
scale,
height,
range,
basePairs) {
  var pxPerLetter = scale(1) - scale(0);
  var mode = _DisplayMode["default"].getDisplayMode(pxPerLetter);
  var showText = _DisplayMode["default"].isText(mode);

  // get adjusted canvas height for drawing letters and rects
  var adjustedHeight = height / window.devicePixelRatio;

  if (mode != _DisplayMode["default"].HIDDEN) {
    ctx.textAlign = 'center';
    ctx.font = _style["default"].TEXT_STYLE(mode, adjustedHeight);

    var previousBase = null;
    var start = range.start(),
    stop = range.stop();
    for (var pos = start; pos <= stop; pos++) {
      var letter = basePairs[pos - start];
      if (letter == '.') continue; // not yet known

      ctx.save();
      ctx.pushObject({ pos: pos, letter: letter });
      ctx.fillStyle = _style["default"].BASE_COLORS[letter];
      if (showText) {
        // We only push objects in the text case as it involves creating a
        // new object & can become a performance issue.
        // 0.5 = centered
        ctx.fillText(letter, scale(1 + 0.5 + pos), adjustedHeight);
      } else {
        if (pxPerLetter >= _style["default"].COVERAGE_MIN_BAR_WIDTH_FOR_GAP) {
          // We want a white space between blocks at this size, so we can see
          // the difference between bases.
          ctx.fillRect(scale(1 + pos) + 0.5, 0, pxPerLetter - 1.5, adjustedHeight);
        } else if (previousBase === letter) {
          // Otherwise, we want runs of colors to be completely solid ...
          ctx.fillRect(scale(1 + pos) - 1.5, 0, pxPerLetter + 1.5, adjustedHeight);
        } else {
          // ... and minimize the amount of smudging and whitespace between
          // bases.
          ctx.fillRect(scale(1 + pos) - 0.5, 0, pxPerLetter + 1.5, adjustedHeight);
        }
      }

      ctx.popObject();
      ctx.restore();
      previousBase = letter;
    }
  }
}var


GenomeTiledCanvas = /*#__PURE__*/function (_TiledCanvas) {_inherits(GenomeTiledCanvas, _TiledCanvas);var _super = _createSuper(GenomeTiledCanvas);



  function GenomeTiledCanvas(source, height) {var _this;_classCallCheck(this, GenomeTiledCanvas);
    _this = _super.call(this);
    _this.source = source;
    // workaround for an issue in PhantomJS where height always comes out to zero.
    _this.height = Math.max(1, height);return _this;
  }_createClass(GenomeTiledCanvas, [{ key: "render", value:

    function render(ctx,
    scale,
    range) {
      // The +/-1 ensures that partially-visible bases on the edge are rendered.
      var genomeRange = {
        contig: range.contig,
        start: Math.max(0, range.start() - 1),
        stop: range.stop() + 1 };

      var basePairs = this.source.getRangeAsString(genomeRange);
      renderGenome(ctx, scale, this.height, _ContigInterval["default"].fromGenomeRange(genomeRange), basePairs);
    } }, { key: "heightForRef", value:

    function heightForRef(ref) {
      return this.height;
    } }, { key: "updateHeight", value:

    function updateHeight(height) {
      this.height = height;
    } }]);return GenomeTiledCanvas;}(_TiledCanvas2["default"]);var


GenomeTrack = /*#__PURE__*/function (_React$Component) {_inherits(GenomeTrack, _React$Component);var _super2 = _createSuper(GenomeTrack);

  // no state, used to make flow happy



  function GenomeTrack(props) {var _this2;_classCallCheck(this, GenomeTrack);
    _this2 = _super2.call(this, props);
    _this2.ref = /*#__PURE__*/_react["default"].createRef();return _this2;
  }_createClass(GenomeTrack, [{ key: "render", value:

    function render() {
      return /*#__PURE__*/_react["default"].createElement("canvas", { ref: this.ref });
    } }, { key: "componentDidMount", value:

    function componentDidMount() {var _this3 = this;
      this.tiles = new GenomeTiledCanvas(this.props.source, this.props.height);

      // Visualize new reference data as it comes in from the network.
      this.props.source.on('newdata', function (range) {
        _this3.tiles.invalidateRange(range);
        _this3.updateVisualization();
      });

      this.updateVisualization();
    } }, { key: "getScale", value:

    function getScale() {
      return _d3utils["default"].getTrackScale(this.props.range, this.props.width);
    } }, { key: "componentDidUpdate", value:

    function componentDidUpdate(prevProps, prevState) {
      if (!(0, _shallowEquals["default"])(prevProps, this.props) ||
      !(0, _shallowEquals["default"])(prevState, this.state)) {
        if (this.props.height != prevProps.height) {
          this.tiles.updateHeight(this.props.height);
          this.tiles.invalidateAll();
        }
        this.updateVisualization();
      }
    } }, { key: "updateVisualization", value:

    function updateVisualization() {
      var canvas = this.ref.current;var _this$props =
      this.props,width = _this$props.width,height = _this$props.height,range = _this$props.range;

      // Hold off until height & width are known.
      if (width === 0) return;

      if (canvas && canvas instanceof Element) {// check for getContext
        if (canvas instanceof HTMLCanvasElement) {// check for sizeCanvas
          _d3utils["default"].sizeCanvas(canvas, width, height);
        }
        var ctx = _dataCanvas["default"].getDataContext(_canvasUtils["default"].getContext(canvas));

        ctx.reset();
        ctx.clearRect(0, 0, ctx.canvas.width, ctx.canvas.height);
        this.tiles.renderToScreen(ctx, _ContigInterval["default"].fromGenomeRange(range), this.getScale());
      } // end typecheck for canvas
    } }]);return GenomeTrack;}(_react["default"].Component);


GenomeTrack.displayName = 'reference';

module.exports = GenomeTrack;