/**
 * Data management for PileupTrack.
 *
 * This class groups paired reads and piles them up.
 *
 * 
 */
'use strict';function _typeof(obj) {"@babel/helpers - typeof";if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {_typeof = function _typeof(obj) {return typeof obj;};} else {_typeof = function _typeof(obj) {return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;};}return _typeof(obj);}




var _AbstractCache2 = _interopRequireDefault(require("./AbstractCache"));


var _underscore = _interopRequireDefault(require("underscore"));
var _ContigInterval = _interopRequireDefault(require("../ContigInterval"));
var _Interval = _interopRequireDefault(require("../Interval"));
var _pileuputils = require("./pileuputils");
var _utils = _interopRequireDefault(require("../utils"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}function _createForOfIteratorHelper(o, allowArrayLike) {var it;if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {if (it) o = it;var i = 0;var F = function F() {};return { s: F, n: function n() {if (i >= o.length) return { done: true };return { done: false, value: o[i++] };}, e: function e(_e) {throw _e;}, f: F };}throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");}var normalCompletion = true,didErr = false,err;return { s: function s() {it = o[Symbol.iterator]();}, n: function n() {var step = it.next();normalCompletion = step.done;return step;}, e: function e(_e2) {didErr = true;err = _e2;}, f: function f() {try {if (!normalCompletion && it["return"] != null) it["return"]();} finally {if (didErr) throw err;}} };}function _unsupportedIterableToArray(o, minLen) {if (!o) return;if (typeof o === "string") return _arrayLikeToArray(o, minLen);var n = Object.prototype.toString.call(o).slice(8, -1);if (n === "Object" && o.constructor) n = o.constructor.name;if (n === "Map" || n === "Set") return Array.from(o);if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);}function _arrayLikeToArray(arr, len) {if (len == null || len > arr.length) len = arr.length;for (var i = 0, arr2 = new Array(len); i < len; i++) {arr2[i] = arr[i];}return arr2;}function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError("Cannot call a class as a function");}}function _defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ("value" in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}function _createClass(Constructor, protoProps, staticProps) {if (protoProps) _defineProperties(Constructor.prototype, protoProps);if (staticProps) _defineProperties(Constructor, staticProps);return Constructor;}function _inherits(subClass, superClass) {if (typeof superClass !== "function" && superClass !== null) {throw new TypeError("Super expression must either be null or a function");}subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } });if (superClass) _setPrototypeOf(subClass, superClass);}function _setPrototypeOf(o, p) {_setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {o.__proto__ = p;return o;};return _setPrototypeOf(o, p);}function _createSuper(Derived) {var hasNativeReflectConstruct = _isNativeReflectConstruct();return function _createSuperInternal() {var Super = _getPrototypeOf(Derived),result;if (hasNativeReflectConstruct) {var NewTarget = _getPrototypeOf(this).constructor;result = Reflect.construct(Super, arguments, NewTarget);} else {result = Super.apply(this, arguments);}return _possibleConstructorReturn(this, result);};}function _possibleConstructorReturn(self, call) {if (call && (_typeof(call) === "object" || typeof call === "function")) {return call;}return _assertThisInitialized(self);}function _assertThisInitialized(self) {if (self === void 0) {throw new ReferenceError("this hasn't been initialised - super() hasn't been called");}return self;}function _isNativeReflectConstruct() {if (typeof Reflect === "undefined" || !Reflect.construct) return false;if (Reflect.construct.sham) return false;if (typeof Proxy === "function") return true;try {Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {}));return true;} catch (e) {return false;}}function _getPrototypeOf(o) {_getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {return o.__proto__ || Object.getPrototypeOf(o);};return _getPrototypeOf(o);}











// Insert sizes within this percentile range will be considered "normal".
var MIN_OUTLIER_PERCENTILE = 0.5;
var MAX_OUTLIER_PERCENTILE = 99.5;
var MAX_INSERT_SIZE = 30000; // ignore inserts larger than this in calculations
var MIN_READS_FOR_OUTLIERS = 500; // minimum reads for reliable stats






// This class provides data management for the visualization, grouping paired
// reads and managing the pileup.
var PileupCache = /*#__PURE__*/function (_AbstractCache) {_inherits(PileupCache, _AbstractCache);var _super = _createSuper(PileupCache);
  // maps groupKey to VisualGroup






  function PileupCache(referenceSource, viewAsPairs) {var _this;_classCallCheck(this, PileupCache);
    _this = _super.call(this, referenceSource);
    _this.viewAsPairs = viewAsPairs;
    _this._insertStats = null;return _this;
  }

  // read name would make a good key, but paired reads from different contigs
  // shouldn't be grouped visually. Hence we use read name + contig.
  _createClass(PileupCache, [{ key: "groupKey", value: function groupKey(read) {
      if (this.viewAsPairs) {
        return "".concat(read.name, ":").concat(read.ref);
      } else {
        return read.getKey();
      }
    }

    // Load a new read into the visualization cache.
    // Calling this multiple times with the same read is a no-op.
  }, { key: "addAlignment", value: function addAlignment(read) {
      this._insertStats = null; // invalidate
      var key = this.groupKey(read),
      range = read.getInterval();

      if (!(key in this.groups)) {
        this.groups[key] = {
          key: key,
          row: -1, // TBD
          insert: null, // TBD
          span: range,
          items: [] };

      }
      var group = this.groups[key];

      if (_underscore["default"].find(group.items, function (a) {return a.read == read;})) {
        return; // we've already got it.
      }

      var opInfo = (0, _pileuputils.getOpInfo)(read, this.referenceSource);
      var visualAlignment = {
        read: read,
        strand: read.getStrand(),
        refLength: range.length(),
        ops: opInfo.ops,
        mismatches: opInfo.mismatches };

      group.items.push(visualAlignment);

      var mateInterval = null;
      if (group.items.length == 1) {
        // This is the first read in the group. Infer its span from its mate properties.
        // TODO: if the mate Alignment is also available, it would be better to use that.
        if (this.viewAsPairs) {
          var mateProps = read.getMateProperties();
          var intervals = [range];
          if (mateProps && mateProps.ref && mateProps.ref == read.ref) {
            mateInterval = new _ContigInterval["default"](mateProps.ref, mateProps.pos, mateProps.pos + range.length());
            intervals.push(mateInterval);
          }
          group = _underscore["default"].extend(group, spanAndInsert(intervals));
        } else {
          group.span = range;
        }

        if (!(read.ref in this.refToPileup)) {
          this.refToPileup[read.ref] = [];
        }
        var pileup = this.refToPileup[read.ref];
        group.row = (0, _pileuputils.addToPileup)(group.span.interval, pileup);
      } else if (group.items.length == 2) {
        // Refine the connector
        mateInterval = group.items[0].read.getInterval();var _spanAndInsert =
        spanAndInsert([range, mateInterval]),span = _spanAndInsert.span,insert = _spanAndInsert.insert;
        group.insert = insert;
        if (insert) {
          group.span = span;
        }
      } else {
        // this must be a chimeric read.
      }
    }

    // Updates reference mismatch information for previously-loaded reads.
  }, { key: "updateMismatches", value: function updateMismatches(range) {
      for (var k in this.groups) {
        var reads = this.groups[k].items;var _iterator = _createForOfIteratorHelper(
        reads),_step;try {for (_iterator.s(); !(_step = _iterator.n()).done;) {var vRead = _step.value;
            var read = vRead.read;
            if (read.getInterval().intersects(range)) {
              var opInfo = (0, _pileuputils.getOpInfo)(read, this.referenceSource);
              vRead.mismatches = opInfo.mismatches;
            }
          }} catch (err) {_iterator.e(err);} finally {_iterator.f();}
      }
    }

    // Re-sort the pileup so that reads overlapping the locus are on top.
  }, { key: "sortReadsAt", value: function sortReadsAt(contig, position) {
      // Strategy: For each pileup row, determine whether it overlaps the locus.
      // Then sort the array indices to get a permutation.
      // Build a new pileup by permuting the old pileup
      // Update all the `row` properties of the relevant visual groups

      var pileup = this.pileupForRef(contig);

      // Find the groups for which an alignment overlaps the locus.
      var groups = _underscore["default"].filter(this.groups,
      function (group) {return _underscore["default"].any(group.items,
        function (a) {return a.read.getInterval().containsLocus(contig, position);});});

      // For each row, find the left-most point (for sorting).
      var rowsOverlapping =
      _underscore["default"].mapObject(_underscore["default"].groupBy(groups, function (g) {return g.row;}),
      function (gs) {return _underscore["default"].min(gs, function (g) {return g.span.start();}).span.start();});

      // Sort groups by whether they overlap, then by their start.
      // TODO: is there an easier way to construct the forward map directly?
      var permutation = _underscore["default"].sortBy(_underscore["default"].range(0, pileup.length),
      function (idx) {return rowsOverlapping[idx] || Infinity;});
      var oldToNew = [];
      permutation.forEach(function (oldIndex, newIndex) {oldToNew[oldIndex] = newIndex;});
      var newPileup = _underscore["default"].range(0, pileup.length).map(function (i) {return pileup[oldToNew[i]];});

      var normRef = contig in this.refToPileup ? contig : _utils["default"].altContigName(contig);
      this.refToPileup[normRef] = newPileup;

      _underscore["default"].each(this.groups, function (g) {
        if (g.span.chrOnContig(contig)) {
          g.row = oldToNew[g.row];
        }
      });
    } }, { key: "getInsertStats", value:

    function getInsertStats() {
      if (this._insertStats) return this._insertStats;
      var inserts = _underscore["default"].map(this.groups,
      function (g) {return g.items[0].read.getInferredInsertSize();}).
      filter(function (x) {return x < MAX_INSERT_SIZE;});
      var insertStats = inserts.length >= MIN_READS_FOR_OUTLIERS ? {
        minOutlierSize: _utils["default"].computePercentile(inserts, MIN_OUTLIER_PERCENTILE),
        maxOutlierSize: _utils["default"].computePercentile(inserts, MAX_OUTLIER_PERCENTILE) } :
      {
        minOutlierSize: 0,
        maxOutlierSize: Infinity };


      this._insertStats = insertStats;
      return insertStats;
    } }]);return PileupCache;}(_AbstractCache2["default"]);


// Helper method for addRead.
// Given 1-2 intervals, compute their span and insert (interval between them).
// For one interval, these are both trivial.
// TODO: what this calls an "insert" is not what most people mean by that.
function spanAndInsert(intervals) {
  if (intervals.length == 1) {
    return { insert: null, span: intervals[0] };
  } else if (intervals.length != 2) {
    throw "Called spanAndInsert with ".concat(intervals.length, " \notin [1, 2]");
  }

  if (!intervals[0].chrOnContig(intervals[1].contig)) {
    return spanAndInsert([intervals[0]]);
  }
  var iv1 = intervals[0].interval,
  iv2 = intervals[1].interval,
  insert = iv1.start < iv2.start ?
  new _Interval["default"](iv1.stop, iv2.start) :
  new _Interval["default"](iv2.stop, iv1.start);

  var span = _Interval["default"].boundingInterval([iv1, iv2]);
  return {
    insert: insert,
    span: new _ContigInterval["default"](intervals[0].contig, span.start, span.stop) };

}

module.exports = PileupCache;