/**
 * Pileup visualization of BAM sources.
 * 
 */
'use strict';function _typeof(obj) {"@babel/helpers - typeof";if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {_typeof = function _typeof(obj) {return typeof obj;};} else {_typeof = function _typeof(obj) {return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;};}return _typeof(obj);}










var _react = _interopRequireDefault(require("react"));
var _shallowEquals = _interopRequireDefault(require("shallow-equals"));
var _underscore = _interopRequireDefault(require("underscore"));

var _scale = _interopRequireDefault(require("../scale"));
var _d3utils = _interopRequireDefault(require("./d3utils"));
var _pileuputils = require("./pileuputils");
var _ContigInterval = _interopRequireDefault(require("../ContigInterval"));
var _DisplayMode = _interopRequireDefault(require("./DisplayMode"));
var _PileupCache = _interopRequireDefault(require("./PileupCache"));
var _TiledCanvas2 = _interopRequireDefault(require("./TiledCanvas"));
var _canvasUtils = _interopRequireDefault(require("./canvas-utils"));
var _dataCanvas = _interopRequireDefault(require("data-canvas"));
var _style = _interopRequireDefault(require("../style"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError("Cannot call a class as a function");}}function _defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ("value" in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}function _createClass(Constructor, protoProps, staticProps) {if (protoProps) _defineProperties(Constructor.prototype, protoProps);if (staticProps) _defineProperties(Constructor, staticProps);return Constructor;}function _inherits(subClass, superClass) {if (typeof superClass !== "function" && superClass !== null) {throw new TypeError("Super expression must either be null or a function");}subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } });if (superClass) _setPrototypeOf(subClass, superClass);}function _setPrototypeOf(o, p) {_setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {o.__proto__ = p;return o;};return _setPrototypeOf(o, p);}function _createSuper(Derived) {var hasNativeReflectConstruct = _isNativeReflectConstruct();return function _createSuperInternal() {var Super = _getPrototypeOf(Derived),result;if (hasNativeReflectConstruct) {var NewTarget = _getPrototypeOf(this).constructor;result = Reflect.construct(Super, arguments, NewTarget);} else {result = Super.apply(this, arguments);}return _possibleConstructorReturn(this, result);};}function _possibleConstructorReturn(self, call) {if (call && (_typeof(call) === "object" || typeof call === "function")) {return call;}return _assertThisInitialized(self);}function _assertThisInitialized(self) {if (self === void 0) {throw new ReferenceError("this hasn't been initialised - super() hasn't been called");}return self;}function _isNativeReflectConstruct() {if (typeof Reflect === "undefined" || !Reflect.construct) return false;if (Reflect.construct.sham) return false;if (typeof Proxy === "function") return true;try {Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {}));return true;} catch (e) {return false;}}function _getPrototypeOf(o) {_getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {return o.__proto__ || Object.getPrototypeOf(o);};return _getPrototypeOf(o);}


var READ_STRAND_ARROW_WIDTH = 5;

// PhantomJS does not support setLineDash.
// Node doesn't even know about the symbol.
var SUPPORTS_DASHES = typeof CanvasRenderingContext2D !== 'undefined' &&
!!CanvasRenderingContext2D.prototype.setLineDash;var

PileupTiledCanvas = /*#__PURE__*/function (_TiledCanvas) {_inherits(PileupTiledCanvas, _TiledCanvas);var _super = _createSuper(PileupTiledCanvas);



  function PileupTiledCanvas(cache, options) {var _this;_classCallCheck(this, PileupTiledCanvas);
    _this = _super.call(this);
    _this.cache = cache;
    _this.options = options;return _this;
  }_createClass(PileupTiledCanvas, [{ key: "update", value:

    function update(newOptions) {
      this.options = newOptions;
    } }, { key: "heightForRef", value:

    function heightForRef(ref) {
      return this.cache.pileupHeightForRef(ref) * (
      _style["default"].READ_HEIGHT + _style["default"].READ_SPACING);
    } }, { key: "render", value:

    function render(ctx,
    scale,
    range) {
      var relaxedRange =
      new _ContigInterval["default"](range.contig, range.start() - 1, range.stop() + 1);
      var vGroups = this.cache.getGroupsOverlapping(relaxedRange);
      var insertStats = this.options.colorByInsert ? this.cache.getInsertStats() : null;
      renderPileup(ctx, scale, relaxedRange, insertStats, this.options.colorByStrand, vGroups);
    } }]);return PileupTiledCanvas;}(_TiledCanvas2["default"]);



// Should the Cigar op be rendered to the screen?
function isRendered(op) {
  return op.op == _pileuputils.CigarOp.MATCH ||
  op.op == _pileuputils.CigarOp.DELETE ||
  op.op == _pileuputils.CigarOp.INSERT;
}

// Render a portion of the pileup into the canvas.
function renderPileup(ctx,
scale,
range,
insertStats,
colorByStrand,
vGroups) {
  // Should mismatched base pairs be shown as blocks of color or as letters?
  var pxPerLetter = scale(1) - scale(0),
  mode = _DisplayMode["default"].getDisplayMode(pxPerLetter),
  showText = _DisplayMode["default"].isText(mode);

  function drawArrow(pos, refLength, top, direction) {
    var left = scale(pos + 1),
    right = scale(pos + refLength + 1),
    bottom = top + _style["default"].READ_HEIGHT,
    // Arrowheads become a distraction as you zoom out and the reads get
    // shorter. They should never be more than 1/6 the read length.
    arrowSize = Math.min(READ_STRAND_ARROW_WIDTH, (right - left) / 6);

    ctx.beginPath();
    if (direction == 'R') {
      ctx.moveTo(left, top);
      ctx.lineTo(right - arrowSize, top);
      ctx.lineTo(right, (top + bottom) / 2);
      ctx.lineTo(right - arrowSize, bottom);
      ctx.lineTo(left, bottom);
    } else {
      ctx.moveTo(right, top);
      ctx.lineTo(left + arrowSize, top);
      ctx.lineTo(left, (top + bottom) / 2);
      ctx.lineTo(left + arrowSize, bottom);
      ctx.lineTo(right, bottom);
    }
    ctx.fill();
  }

  function drawSegment(op, y, vRead) {
    switch (op.op) {
      case _pileuputils.CigarOp.MATCH:
        if (op.arrow) {
          drawArrow(op.pos, op.length, y, op.arrow);
        } else {
          var x = scale(op.pos + 1);
          ctx.fillRect(x, y, scale(op.pos + op.length + 1) - x, _style["default"].READ_HEIGHT);
        }
        break;

      case _pileuputils.CigarOp.DELETE:
        var x1 = scale(op.pos + 1),
        x2 = scale(op.pos + 1 + op.length),
        yp = y + _style["default"].READ_HEIGHT / 2 - 0.5;
        ctx.save();
        ctx.fillStyle = _style["default"].DELETE_COLOR;
        ctx.fillRect(x1, yp, x2 - x1, 1);
        ctx.restore();
        break;

      case _pileuputils.CigarOp.INSERT:
        ctx.save();
        ctx.fillStyle = _style["default"].INSERT_COLOR;
        var x0 = scale(op.pos + 1) - 2, // to cover a bit of the previous segment
        y1 = y - 1,
        y2 = y + _style["default"].READ_HEIGHT + 2;
        ctx.fillRect(x0, y1, 1, y2 - y1);
        ctx.restore();
        break;}

  }

  function drawAlignment(vRead, y) {
    ctx.pushObject(vRead);
    ctx.save();
    if (colorByStrand) {
      ctx.fillStyle = vRead.strand == '+' ?
      _style["default"].ALIGNMENT_PLUS_STRAND_COLOR :
      _style["default"].ALIGNMENT_MINUS_STRAND_COLOR;
    }
    vRead.ops.forEach(function (op) {
      if (isRendered(op)) {
        drawSegment(op, y, vRead);
      }
    });
    vRead.mismatches.forEach(function (bp) {return renderMismatch(bp, y);});
    ctx.restore();
    ctx.popObject();
  }

  function drawGroup(vGroup) {
    ctx.save();
    if (insertStats && vGroup.insert) {
      var len = vGroup.span.length();
      if (len < insertStats.minOutlierSize) {
        ctx.fillStyle = 'blue';
      } else if (len > insertStats.maxOutlierSize) {
        ctx.fillStyle = 'red';
      } else {
        ctx.fillStyle = _style["default"].ALIGNMENT_COLOR;
      }
    } else {
      ctx.fillStyle = _style["default"].ALIGNMENT_COLOR;
    }
    var y = (0, _pileuputils.yForRow)(vGroup.row);
    ctx.pushObject(vGroup);
    if (vGroup.insert) {
      var span = vGroup.insert,
      x1 = scale(span.start + 1),
      x2 = scale(span.stop + 1);
      ctx.fillRect(x1, y + _style["default"].READ_HEIGHT / 2 - 0.5, x2 - x1, 1);
    }
    vGroup.items.forEach(function (vRead) {return drawAlignment(vRead, y);});
    ctx.popObject();
    ctx.restore();
  }

  function renderMismatch(bp, y) {
    // This can happen if the mismatch is in a different tile, for example.
    if (!range.interval.contains(bp.pos)) return;

    ctx.pushObject(bp);
    ctx.save();
    ctx.fillStyle = _style["default"].BASE_COLORS[bp.basePair];
    ctx.globalAlpha = opacityForQuality(bp.quality);
    ctx.textAlign = 'center';
    if (showText) {
      // 0.5 = centered
      ctx.fillText(bp.basePair, scale(1 + 0.5 + bp.pos), y + _style["default"].READ_HEIGHT - 2);
    } else {
      ctx.fillRect(scale(1 + bp.pos), y, pxPerLetter - 1, _style["default"].READ_HEIGHT);
    }
    ctx.restore();
    ctx.popObject();
  }

  ctx.font = _style["default"].TEXT_STYLE(_DisplayMode["default"].TIGHT, _style["default"].READ_HEIGHT - 1);
  vGroups.forEach(function (vGroup) {return drawGroup(vGroup);});
}

// This is adapted from IGV.
var MIN_Q = 5, // these are Phred-scaled scores
MAX_Q = 20,
Q_SCALE = _scale["default"].linear().
domain([MIN_Q, MAX_Q]).
range([0.1, 0.9]).
clamp(true); // clamp output to [0.1, 0.9]
function opacityForQuality(quality) {
  var alpha = Q_SCALE(quality);

  // Round alpha to nearest 0.1
  alpha = Math.round(alpha * 10 + 0.5) / 10.0;
  return Math.min(1.0, alpha);
}var

PileupTrack = /*#__PURE__*/function (_React$Component) {_inherits(PileupTrack, _React$Component);var _super2 = _createSuper(PileupTrack);









  function PileupTrack(props) {var _this2;_classCallCheck(this, PileupTrack);
    _this2 = _super2.call(this, props);
    _this2.ref = /*#__PURE__*/_react["default"].createRef();
    _this2.state = {
      networkStatus: null };return _this2;

  }_createClass(PileupTrack, [{ key: "render", value:

    function render() {
      // These styles allow vertical scrolling to see the full pileup.
      // Adding a vertical scrollbar shrinks the visible area, but we have to act
      // as though it doesn't, since adjusting the scale would put it out of sync
      // with other tracks.
      var containerStyles = {
        'height': '100%' };


      var statusEl = null,
      networkStatus = this.state.networkStatus;
      if (networkStatus) {
        var message = this.formatStatus(networkStatus);
        statusEl = /*#__PURE__*/
        _react["default"].createElement("div", { className: "network-status" }, /*#__PURE__*/
        _react["default"].createElement("div", { className: "network-status-message" }, "Loading alignments\u2026 (",
        message, ")"));



      }

      return /*#__PURE__*/(
        _react["default"].createElement("div", null,
        statusEl, /*#__PURE__*/
        _react["default"].createElement("div", { style: containerStyles }, /*#__PURE__*/
        _react["default"].createElement("canvas", { ref: this.ref, onClick: this.handleClick.bind(this) }))));



    } }, { key: "formatStatus", value:

    function formatStatus(status) {
      if (status.numRequests) {
        var pluralS = status.numRequests > 1 ? 's' : '';
        return "issued ".concat(status.numRequests, " request").concat(pluralS);
      } else if (status.status) {
        return status.status;
      }
      throw 'invalid';
    } }, { key: "componentDidMount", value:

    function componentDidMount() {var _this3 = this;
      this.cache = new _PileupCache["default"](this.props.referenceSource, this.props.options.viewAsPairs);
      this.tiles = new PileupTiledCanvas(this.cache, this.props.options);

      this.props.source.on('newdata', function (range) {
        _this3.updateReads(range);
        _this3.tiles.invalidateRange(range);
        _this3.updateVisualization();
      });
      this.props.referenceSource.on('newdata', function (range) {
        _this3.cache.updateMismatches(range);
        _this3.tiles.invalidateRange(range);
        _this3.updateVisualization();
      });
      this.props.source.on('networkprogress', function (e) {
        _this3.setState({ networkStatus: e });
      });
      this.props.source.on('networkdone', function (e) {
        _this3.setState({ networkStatus: null });
      });

      this.updateVisualization();
    } }, { key: "getScale", value:

    function getScale() {
      return _d3utils["default"].getTrackScale(this.props.range, this.props.width);
    } }, { key: "componentDidUpdate", value:

    function componentDidUpdate(prevProps, prevState) {
      var shouldUpdate = false;
      if (this.props.options != prevProps.options) {
        this.handleOptionsChange(prevProps.options);
        shouldUpdate = true;
      }

      if (!(0, _shallowEquals["default"])(this.props, prevProps) ||
      !(0, _shallowEquals["default"])(this.state, prevState) ||
      shouldUpdate) {
        this.updateVisualization();
      }
    } }, { key: "handleOptionsChange", value:

    function handleOptionsChange(oldOpts) {
      this.tiles.invalidateAll();

      if (oldOpts.viewAsPairs != this.props.options.viewAsPairs) {
        this.cache = new _PileupCache["default"](this.props.referenceSource, this.props.options.viewAsPairs);
        this.tiles = new PileupTiledCanvas(this.cache, this.props.options);
        this.updateReads(_ContigInterval["default"].fromGenomeRange(this.props.range));
      } else if (oldOpts.colorByInsert != this.props.options.colorByInsert) {
        this.tiles.update(this.props.options);
        this.tiles.invalidateAll();
        this.updateVisualization();
      } else if (oldOpts.hideAlignments != this.props.options.hideAlignments) {
        this.tiles.update(this.props.options);
        this.updateVisualization();
      }

      if (oldOpts.sort != this.props.options.sort) {
        this.handleSort();
      }
    }

    // Load new reads into the visualization cache.
  }, { key: "updateReads", value: function updateReads(range) {var _this4 = this;
      var anyBefore = this.cache.anyGroupsOverlapping(range);
      this.props.source.getFeaturesInRange(range).
      forEach(function (read) {return _this4.cache.addAlignment(read);});

      if (!anyBefore && this.cache.anyGroupsOverlapping(range)) {
        // If these are the first reads to be shown in the visible range,
        // then sort them to highlight reads in the center.
        this.handleSort();
      }
    }

    // Update the visualization to reflect the cached reads &
    // currently-visible range.
  }, { key: "updateVisualization", value: function updateVisualization() {
      var canvas = this.ref.current;
      var width = this.props.width;

      // Hold off until height & width are known.
      if (width === 0) return;

      var ctx = _canvasUtils["default"].getContext(canvas);
      var dtx = _dataCanvas["default"].getDataContext(ctx);

      this.renderScene(dtx);

    } }, { key: "renderScene", value:

    function renderScene(ctx) {
      var genomeRange = this.props.range,
      range = new _ContigInterval["default"](genomeRange.contig, genomeRange.start, genomeRange.stop),
      scale = this.getScale(),
      width = this.props.width;
      var canvas = this.ref.current;

      ctx.reset();
      ctx.clearRect(0, 0, ctx.canvas.width, ctx.canvas.height);

      // get parent of canvas
      // The typecasts through `any` are to fool flow.
      var parent = _d3utils["default"].findParent(canvas, "pileup");

      if (this.props.options.hideAlignments) {
        _d3utils["default"].sizeCanvas(canvas, width, 60);
        // if alignments are hidden, set parent div to 60px to leave room for gear
        if (parent) parent.style.minHeight = "60px";
        return;
      }

      // unset minimum hight for viewing alignments
      if (parent) parent.style.minHeight = "";

      // Height can only be computed after the pileup has been updated.
      var height = (0, _pileuputils.yForRow)(this.cache.pileupHeightForRef(this.props.range.contig));

      _d3utils["default"].sizeCanvas(canvas, width, height);

      this.tiles.renderToScreen(ctx, range, scale);

      // TODO: the center line should go above alignments, but below mismatches
      this.renderCenterLine(ctx, range, scale);

      // This is a hack to mitigate #350
      var el = _d3utils["default"].findParent(this.ref.current, 'track-content');
      if (el) el.scrollLeft = 0;
    }

    // Draw the center line(s), which orient the user
  }, { key: "renderCenterLine", value: function renderCenterLine(ctx,
    range,
    scale) {
      var midPoint = Math.floor((range.stop() + range.start()) / 2),
      rightLineX = Math.ceil(scale(midPoint + 1)),
      leftLineX = Math.floor(scale(midPoint)),
      height = ctx.canvas.height;
      ctx.save();
      ctx.lineWidth = 1;
      if (SUPPORTS_DASHES) {
        ctx.setLineDash([5, 5]);
      }
      if (rightLineX - leftLineX < 3) {
        // If the lines are very close, then just draw a center line.
        var midX = Math.round((leftLineX + rightLineX) / 2);
        _canvasUtils["default"].drawLine(ctx, midX - 0.5, 0, midX - 0.5, height);
      } else {
        _canvasUtils["default"].drawLine(ctx, leftLineX - 0.5, 0, leftLineX - 0.5, height);
        _canvasUtils["default"].drawLine(ctx, rightLineX - 0.5, 0, rightLineX - 0.5, height);
      }
      ctx.restore();
    } }, { key: "handleSort", value:

    function handleSort() {var _this$props$range =
      this.props.range,start = _this$props$range.start,stop = _this$props$range.stop,
      middle = (start + stop) / 2;
      this.cache.sortReadsAt(this.props.range.contig, middle);
      this.tiles.invalidateAll();
      this.updateVisualization();
    } }, { key: "handleClick", value:

    function handleClick(reactEvent) {
      var ev = reactEvent.nativeEvent,
      x = ev.offsetX,
      y = ev.offsetY;
      var ctx = _canvasUtils["default"].getContext(this.ref.current);
      var trackingCtx = new _dataCanvas["default"].ClickTrackingContext(ctx, x, y);

      var genomeRange = this.props.range,
      range = new _ContigInterval["default"](genomeRange.contig, genomeRange.start, genomeRange.stop),
      scale = this.getScale(),
      // If click-tracking gets slow, this range could be narrowed to one
      // closer to the click coordinate, rather than the whole visible range.
      vGroups = this.cache.getGroupsOverlapping(range);

      renderPileup(trackingCtx, scale, range, null, false, vGroups);
      var vRead = _underscore["default"].find(trackingCtx.hits[0], function (hit) {return hit.read;});
      var alert = window.alert || console.log;
      if (vRead) {
        alert(vRead.read.debugString());
      }
    } }]);return PileupTrack;}(_react["default"].Component);


PileupTrack.displayName = 'pileup';
PileupTrack.defaultOptions = {
  viewAsPairs: false,
  colorByInsert: true,
  colorByStrand: false,
  hideAlignments: false };


PileupTrack.getOptionsMenu = function (options) {
  return [
  { key: 'view-pairs', label: 'View as pairs', checked: options.viewAsPairs },
  '-',
  { key: 'color-insert', label: 'Color by insert size', checked: options.colorByInsert },
  { key: 'color-strand', label: 'Color by strand', checked: options.colorByStrand },
  { key: 'hide-alignments', label: 'Hide alignments', checked: options.hideAlignments },
  '-',
  { key: 'sort', label: 'Sort alignments' }];

};

var messageId = 1;

PileupTrack.handleSelectOption = function (item, oldOptions) {
  var opts = _underscore["default"].clone(oldOptions);
  if (item.key == 'view-pairs') {
    opts.viewAsPairs = !opts.viewAsPairs;
    return opts;
  } else if (item.key == 'color-insert') {
    opts.colorByInsert = !opts.colorByInsert;
    if (opts.colorByInsert) opts.colorByStrand = false;
    return opts;
  } else if (item.key == 'color-strand') {
    opts.colorByStrand = !opts.colorByStrand;
    if (opts.colorByStrand) opts.colorByInsert = false;
    return opts;
  } else if (item.key == 'hide-alignments') {
    opts.hideAlignments = !opts.hideAlignments;
    return opts;
  } else if (item.key == 'sort') {
    opts.sort = messageId++;
    return opts;
  }
  return oldOptions; // no change
};


module.exports = PileupTrack;