/**
 * This class allows testing with extremely large files, without the need to
 * store large files in the git repo.
 *
 * It stores subsets of a file on disk, then maps these back to the original
 * portions of the file.
 *
 * Recommended usage:
 * - In your test, use RecordedRemoteFile with the real remote file.
 *   At the end of the test, add:
 *      console.log(JSON.stringify(remoteFile.getRequests()))
 *   and copy the output to the clipboard.
 * - Generate a mapped file using scripts/generate_mapped_file.py:
 *   pbpaste | ./scripts/generate_mapped_file.py http://path/to/url
 * - Replace RecordedRemoteFile in the test with MappedRemoteFile.
 *
 * 
 */
'use strict';function _typeof(obj) {"@babel/helpers - typeof";if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {_typeof = function _typeof(obj) {return typeof obj;};} else {_typeof = function _typeof(obj) {return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;};}return _typeof(obj);}

var _q = _interopRequireDefault(require("q"));

var _RemoteFile2 = _interopRequireDefault(require("../main/RemoteFile"));
var _Interval = _interopRequireDefault(require("../main/Interval"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}function _slicedToArray(arr, i) {return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();}function _nonIterableRest() {throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");}function _unsupportedIterableToArray(o, minLen) {if (!o) return;if (typeof o === "string") return _arrayLikeToArray(o, minLen);var n = Object.prototype.toString.call(o).slice(8, -1);if (n === "Object" && o.constructor) n = o.constructor.name;if (n === "Map" || n === "Set") return Array.from(o);if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);}function _arrayLikeToArray(arr, len) {if (len == null || len > arr.length) len = arr.length;for (var i = 0, arr2 = new Array(len); i < len; i++) {arr2[i] = arr[i];}return arr2;}function _iterableToArrayLimit(arr, i) {if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return;var _arr = [];var _n = true;var _d = false;var _e = undefined;try {for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {_arr.push(_s.value);if (i && _arr.length === i) break;}} catch (err) {_d = true;_e = err;} finally {try {if (!_n && _i["return"] != null) _i["return"]();} finally {if (_d) throw _e;}}return _arr;}function _arrayWithHoles(arr) {if (Array.isArray(arr)) return arr;}function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError("Cannot call a class as a function");}}function _defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ("value" in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}function _createClass(Constructor, protoProps, staticProps) {if (protoProps) _defineProperties(Constructor.prototype, protoProps);if (staticProps) _defineProperties(Constructor, staticProps);return Constructor;}function _get(target, property, receiver) {if (typeof Reflect !== "undefined" && Reflect.get) {_get = Reflect.get;} else {_get = function _get(target, property, receiver) {var base = _superPropBase(target, property);if (!base) return;var desc = Object.getOwnPropertyDescriptor(base, property);if (desc.get) {return desc.get.call(receiver);}return desc.value;};}return _get(target, property, receiver || target);}function _superPropBase(object, property) {while (!Object.prototype.hasOwnProperty.call(object, property)) {object = _getPrototypeOf(object);if (object === null) break;}return object;}function _inherits(subClass, superClass) {if (typeof superClass !== "function" && superClass !== null) {throw new TypeError("Super expression must either be null or a function");}subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } });if (superClass) _setPrototypeOf(subClass, superClass);}function _setPrototypeOf(o, p) {_setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {o.__proto__ = p;return o;};return _setPrototypeOf(o, p);}function _createSuper(Derived) {var hasNativeReflectConstruct = _isNativeReflectConstruct();return function _createSuperInternal() {var Super = _getPrototypeOf(Derived),result;if (hasNativeReflectConstruct) {var NewTarget = _getPrototypeOf(this).constructor;result = Reflect.construct(Super, arguments, NewTarget);} else {result = Super.apply(this, arguments);}return _possibleConstructorReturn(this, result);};}function _possibleConstructorReturn(self, call) {if (call && (_typeof(call) === "object" || typeof call === "function")) {return call;}return _assertThisInitialized(self);}function _assertThisInitialized(self) {if (self === void 0) {throw new ReferenceError("this hasn't been initialised - super() hasn't been called");}return self;}function _isNativeReflectConstruct() {if (typeof Reflect === "undefined" || !Reflect.construct) return false;if (Reflect.construct.sham) return false;if (typeof Proxy === "function") return true;try {Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {}));return true;} catch (e) {return false;}}function _getPrototypeOf(o) {_getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {return o.__proto__ || Object.getPrototypeOf(o);};return _getPrototypeOf(o);}var

MappedRemoteFile = /*#__PURE__*/function (_RemoteFile) {_inherits(MappedRemoteFile, _RemoteFile);var _super = _createSuper(MappedRemoteFile);


  function MappedRemoteFile(url, maps) {var _this;_classCallCheck(this, MappedRemoteFile);
    _this = _super.call(this, url);
    _this.maps = maps.map(function (_ref) {var _ref2 = _slicedToArray(_ref, 2),x = _ref2[0],y = _ref2[1];return new _Interval["default"](x, y);});
    for (var i = 1; i < _this.maps.length; i++) {
      var m0 = _this.maps[i - 1],
      m1 = _this.maps[i];
      if (m0.stop >= m1.start) throw 'Invalid maps';
    }return _this;
  }_createClass(MappedRemoteFile, [{ key: "getFromNetwork", value:

    function getFromNetwork(start, stop) {var _this2 = this;
      // Translate start/stop (which are offsets into the large file) into
      // offsets in the smaller, realized file.
      var originalRequest = new _Interval["default"](start, stop),
      request = null;
      var offset = 0;
      for (var i = 0; i < this.maps.length; i++) {
        var m = this.maps[i];
        if (m.containsInterval(originalRequest)) {
          request = new _Interval["default"](offset + (start - m.start),
          offset + (stop - m.start));
          break;
        } else {
          offset += m.length();
        }
      }

      if (request) {
        return _get(_getPrototypeOf(MappedRemoteFile.prototype), "getFromNetwork", this).call(this, request.start, request.stop).then(function (buf) {
          // RemoteFile may discover the mapped file length from this request.
          // This results in incorrect behavior, so we force it to forget.
          _this2.fileLength = -1;
          return buf;
        });
      } else {
        return _q["default"].reject("Request for ".concat(originalRequest.toString(), " is not mapped in ").concat(this.url));
      }
    } }, { key: "getAll", value:

    function getAll() {
      return _q["default"].reject('Not implemented');
    } }, { key: "getSize", value:

    function getSize() {
      return _q["default"].reject('Not implemented');
    } }]);return MappedRemoteFile;}(_RemoteFile2["default"]);


module.exports = MappedRemoteFile;