
'use strict';

var _chai = require("chai");

var _bedtools = _interopRequireDefault(require("../../main/data/bedtools"));
var _Interval = _interopRequireDefault(require("../../main/Interval"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}

describe('bedtools', function () {
  describe('splitCodingExons', function () {
    var splitCodingExons = _bedtools["default"].splitCodingExons;
    var CodingInterval = _bedtools["default"].CodingInterval;

    it('should split one exon', function (done) {
      var exon = new _Interval["default"](10, 20);

      (0, _chai.expect)(splitCodingExons([exon], new _Interval["default"](13, 17))).to.deep.equal([
      new CodingInterval(10, 12, false),
      new CodingInterval(13, 17, true),
      new CodingInterval(18, 20, false)]);


      (0, _chai.expect)(splitCodingExons([exon], new _Interval["default"](5, 15))).to.deep.equal([
      new CodingInterval(10, 15, true),
      new CodingInterval(16, 20, false)]);


      (0, _chai.expect)(splitCodingExons([exon], new _Interval["default"](15, 25))).to.deep.equal([
      new CodingInterval(10, 14, false),
      new CodingInterval(15, 20, true)]);


      (0, _chai.expect)(splitCodingExons([exon], new _Interval["default"](10, 15))).to.deep.equal([
      new CodingInterval(10, 15, true),
      new CodingInterval(16, 20, false)]);


      (0, _chai.expect)(splitCodingExons([exon], new _Interval["default"](15, 20))).to.deep.equal([
      new CodingInterval(10, 14, false),
      new CodingInterval(15, 20, true)]);

      done();
    });

    it('should handle purely coding or non-coding exons', function (done) {
      var exon = new _Interval["default"](10, 20);

      (0, _chai.expect)(splitCodingExons([exon], new _Interval["default"](0, 9))).to.deep.equal([
      new CodingInterval(10, 20, false)]);

      (0, _chai.expect)(splitCodingExons([exon], new _Interval["default"](21, 25))).to.deep.equal([
      new CodingInterval(10, 20, false)]);

      (0, _chai.expect)(splitCodingExons([exon], new _Interval["default"](10, 20))).to.deep.equal([
      new CodingInterval(10, 20, true)]);

      done();
    });
  });
});