
'use strict';

var _chai = require("chai");

var _ContigInterval = _interopRequireDefault(require("../../main/ContigInterval"));
var _GA4GHVariantJson = _interopRequireDefault(require("../../main/json/GA4GHVariantJson"));
var _RemoteFile = _interopRequireDefault(require("../../main/RemoteFile"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}

describe('GA4GHVariantJson', function () {
  var json;

  before(function () {
    return new _RemoteFile["default"]('/test-data/variants.ga4gh.chr1.10000-11000.json').getAllString().then(function (data) {
      json = data;
    });
  });

  it('should filter variants from json', function (done) {

    var source = _GA4GHVariantJson["default"].create(json);

    var requestInterval = new _ContigInterval["default"]('1', 10000, 10500);

    var variants = source.getVariantsInRange(requestInterval);
    (0, _chai.expect)(variants).to.have.length(3);
    done();

  });

  it('should filter genotypes from json', function (done) {

    var source = _GA4GHVariantJson["default"].create(json);

    var requestInterval = new _ContigInterval["default"]('1', 10000, 10500);

    var variants = source.getGenotypesInRange(requestInterval);
    (0, _chai.expect)(variants).to.have.length(3);
    done();

  });

  it('should get call names from json', function (done) {

    var source = _GA4GHVariantJson["default"].create(json);

    source.getCallNames().then(function (samples) {
      (0, _chai.expect)(samples).to.have.length(1);
      done();
    });
  });

  it('should not fail on empty json string', function (done) {

    var source = _GA4GHVariantJson["default"].create("{}");

    var requestInterval = new _ContigInterval["default"]('1', 10, 20);

    var variants = source.getVariantsInRange(requestInterval);
    (0, _chai.expect)(variants).to.have.length(0);
    done();

  });

  it('should not fail when there are no variants', function (done) {

    var source = _GA4GHVariantJson["default"].create("{\"variants\": []}");

    var requestInterval = new _ContigInterval["default"]('1', 10, 20);

    var variants = source.getVariantsInRange(requestInterval);
    (0, _chai.expect)(variants).to.have.length(0);
    done();

  });

});