
'use strict';

var _chai = require("chai");

var _BigBed = _interopRequireDefault(require("../../main/data/BigBed"));
var _BigBedDataSource = _interopRequireDefault(require("../../main/sources/BigBedDataSource"));
var _ContigInterval = _interopRequireDefault(require("../../main/ContigInterval"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}

describe('BigBedDataSource', function () {
  function getTestSource() {
    // See test/data/README.md
    return _BigBedDataSource["default"].createFromBigBedFile(
    new _BigBed["default"]('/test-data/ensembl.chr17.bb'));
  }

  it('should extract features in a range', function (done) {
    this.timeout(5000);
    var source = getTestSource();

    // No genes fetched initially
    var tp53range = new _ContigInterval["default"]('chr17', 7512444, 7517300);
    var tp53 = source.getFeaturesInRange(tp53range);
    (0, _chai.expect)(tp53).to.deep.equal([]);

    // Fetching that one gene should cache its entire block.
    source.on('newdata', function () {
      var tp53s = source.getFeaturesInRange(tp53range);
      (0, _chai.expect)(tp53s).to.have.length(1);

      var tp53 = tp53s[0];
      (0, _chai.expect)(tp53.name).to.equal('TP53');
      (0, _chai.expect)(tp53.exons).to.have.length(11);
      done();
    });
    source.rangeChanged({
      contig: tp53range.contig,
      start: tp53range.start(),
      stop: tp53range.stop() });

  });
});