
'use strict';

var _chai = require("chai");

var _CytoBandDataSource = _interopRequireDefault(require("../../main/sources/CytoBandDataSource"));
var _cytoBand = require("../../main/data/cytoBand");
var _RemoteFile = _interopRequireDefault(require("../../main/RemoteFile"));
var _ContigInterval = _interopRequireDefault(require("../../main/ContigInterval"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}

describe('CytoBandDataSource', function () {
  function getTestSource() {
    // cytoband file downloaded from
    // http://hgdownload.cse.ucsc.edu/goldenpath/hg19/database/cytoBand.txt.gz
    var f = new _cytoBand.CytoBandFile(new _RemoteFile["default"]('/test-data/cytoBand.txt.gz'));
    return _CytoBandDataSource["default"].createFromCytoBandFile(f);
  }

  it('should fetch chromsome', function (done) {
    var source = getTestSource();
    var range = new _ContigInterval["default"]('chr22', 0, 3);
    // Before data has been fetched, all base pairs are null.
    (0, _chai.expect)(source.getFeaturesInRange(range)).to.deep.equal([undefined]);

    source.on('newdata', function () {
      var chr = source.getFeaturesInRange(range)[0];
      (0, _chai.expect)(chr.name).to.equal('chr22');
      (0, _chai.expect)(chr.bands).to.have.length(16);
      (0, _chai.expect)(chr.bands[0].name).to.equal('p13');
      (0, _chai.expect)(chr.bands[0].value).to.equal('gvar');
      done();
    });
    source.rangeChanged({ contig: range.contig, start: range.start(), stop: range.stop() });
  });

  it('should allow a mix of chr and non-chr', function (done) {
    var source = getTestSource();
    var chrRange = { contig: 'chr1', start: 0, stop: 3 };

    var range = new _ContigInterval["default"]('1', 0, 3);

    source.on('newdata', function () {
      var chr = source.getFeaturesInRange(range)[0];
      (0, _chai.expect)(chr.name).to.equal('chr1');
      (0, _chai.expect)(chr.bands).to.have.length(63);
      (0, _chai.expect)(chr.bands[0].name).to.equal('p36.33');
      (0, _chai.expect)(chr.bands[0].value).to.equal('gneg');
      done();
    });
    source.rangeChanged(chrRange);
  });
});