/**
 * This tests whether feature information is being shown/drawn correctly
 * in the track.
 *
 * 
 */
'use strict';

var _pileup = _interopRequireDefault(require("../../main/pileup"));
var _dataCanvas = _interopRequireDefault(require("data-canvas"));
var _async = require("../async");
var _chai = require("chai");
var _RemoteFile = _interopRequireDefault(require("../../main/RemoteFile"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}

describe('IdiogramTrack', function () {
  var testDiv = document.getElementById('testdiv');
  if (!testDiv) throw new Error("Failed to match: testdiv");

  var drawnObjects = _dataCanvas["default"].RecordingContext.drawnObjects;

  function ready() {
    return testDiv.querySelector('.idiogram canvas') !== null &&
    testDiv.querySelector('.idiogram canvas') !== undefined &&
    drawnObjects(testDiv, '.idiogram').length > 0;
  }

  beforeEach(function () {
    testDiv.style.width = '800px';
    _dataCanvas["default"].RecordingContext.recordAll();
  });

  afterEach(function () {
    _dataCanvas["default"].RecordingContext.reset();
    // avoid pollution between tests.
    testDiv.innerHTML = '';
  });

  describe('json features', function () {
    var json;

    before(function () {
      return new _RemoteFile["default"]('/test-data/gstained_chromosomes_data.json').getAllString().then(function (data) {
        json = data;
      });
    });

    it('should render idiogram with json', function () {
      var p = _pileup["default"].create(testDiv, {
        range: { contig: 'chr17', start: 7500000, stop: 7500500 },
        tracks: [
        {
          viz: _pileup["default"].viz.genome(),
          data: _pileup["default"].formats.twoBit({
            url: '/test-data/test.2bit' }),

          isReference: true },

        {
          viz: _pileup["default"].viz.idiogram(),
          data: _pileup["default"].formats.idiogramJson(json),
          name: 'Idiogram' }] });




      return (0, _async.waitFor)(ready, 2000).
      then(function () {
        var bands = drawnObjects(testDiv, '.idiogram');
        (0, _chai.expect)(bands).to.have.length(24);
        p.destroy();
      });
    });
  });

  describe('cytoband features', function () {

    it('should render idiogram with gzipped file', function () {
      var p = _pileup["default"].create(testDiv, {
        range: { contig: 'chr1', start: 7500000, stop: 7500500 },
        tracks: [
        {
          viz: _pileup["default"].viz.genome(),
          data: _pileup["default"].formats.twoBit({
            url: '/test-data/test.2bit' }),

          isReference: true },

        {
          viz: _pileup["default"].viz.idiogram(),
          data: _pileup["default"].formats.cytoBand('/test-data/cytoBand.txt.gz'),
          name: 'Idiogram' }] });




      return (0, _async.waitFor)(ready, 2000).
      then(function () {
        var bands = drawnObjects(testDiv, '.idiogram');
        (0, _chai.expect)(bands).to.have.length(63);
        p.destroy();
      });
    });

  });
});