/**
 * This tests that pileup mismatches are rendered correctly, regardless of the
 * order in which the alignment and reference data come off the network.
 * 
 */
'use strict';function _typeof(obj) {"@babel/helpers - typeof";if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {_typeof = function _typeof(obj) {return typeof obj;};} else {_typeof = function _typeof(obj) {return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;};}return _typeof(obj);}



var _chai = require("chai");

var _q = _interopRequireDefault(require("q"));
var _underscore = _interopRequireDefault(require("underscore"));

var _pileup = _interopRequireDefault(require("../../main/pileup"));
var _TwoBit2 = _interopRequireDefault(require("../../main/data/TwoBit"));
var _TwoBitDataSource = _interopRequireDefault(require("../../main/sources/TwoBitDataSource"));
var _bam = _interopRequireDefault(require("../../main/data/bam"));
var _BamDataSource = _interopRequireDefault(require("../../main/sources/BamDataSource"));
var _RemoteFile = _interopRequireDefault(require("../../main/RemoteFile"));
var _MappedRemoteFile = _interopRequireDefault(require("../MappedRemoteFile"));
var _ContigInterval = _interopRequireDefault(require("../../main/ContigInterval"));
var _async = require("../async");
var _dataCanvas = _interopRequireDefault(require("data-canvas"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError("Cannot call a class as a function");}}function _defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ("value" in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}function _createClass(Constructor, protoProps, staticProps) {if (protoProps) _defineProperties(Constructor.prototype, protoProps);if (staticProps) _defineProperties(Constructor, staticProps);return Constructor;}function _inherits(subClass, superClass) {if (typeof superClass !== "function" && superClass !== null) {throw new TypeError("Super expression must either be null or a function");}subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } });if (superClass) _setPrototypeOf(subClass, superClass);}function _setPrototypeOf(o, p) {_setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {o.__proto__ = p;return o;};return _setPrototypeOf(o, p);}function _createSuper(Derived) {var hasNativeReflectConstruct = _isNativeReflectConstruct();return function _createSuperInternal() {var Super = _getPrototypeOf(Derived),result;if (hasNativeReflectConstruct) {var NewTarget = _getPrototypeOf(this).constructor;result = Reflect.construct(Super, arguments, NewTarget);} else {result = Super.apply(this, arguments);}return _possibleConstructorReturn(this, result);};}function _possibleConstructorReturn(self, call) {if (call && (_typeof(call) === "object" || typeof call === "function")) {return call;}return _assertThisInitialized(self);}function _assertThisInitialized(self) {if (self === void 0) {throw new ReferenceError("this hasn't been initialised - super() hasn't been called");}return self;}function _isNativeReflectConstruct() {if (typeof Reflect === "undefined" || !Reflect.construct) return false;if (Reflect.construct.sham) return false;if (typeof Proxy === "function") return true;try {Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {}));return true;} catch (e) {return false;}}function _getPrototypeOf(o) {_getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {return o.__proto__ || Object.getPrototypeOf(o);};return _getPrototypeOf(o);}


// This is like TwoBit, but allows a controlled release of sequence data.
var FakeTwoBit = /*#__PURE__*/function (_TwoBit) {_inherits(FakeTwoBit, _TwoBit);var _super = _createSuper(FakeTwoBit);


  function FakeTwoBit(remoteFile) {var _this;_classCallCheck(this, FakeTwoBit);
    _this = _super.call(this, remoteFile);
    _this.deferred = _q["default"].defer();return _this;
  }_createClass(FakeTwoBit, [{ key: "getFeaturesInRange", value:

    function getFeaturesInRange(contig, start, stop) {
      (0, _chai.expect)(contig).to.equal('chr17');
      (0, _chai.expect)(start).to.equal(7500000);
      (0, _chai.expect)(stop).to.equal(7510000);
      return this.deferred.promise;
    } }, { key: "release", value:

    function release(sequence) {
      this.deferred.resolve(sequence);
    } }]);return FakeTwoBit;}(_TwoBit2["default"]);


// This is like Bam, but allows a controlled release of one batch of alignments.
var FakeBam = /*#__PURE__*/function (_Bam) {_inherits(FakeBam, _Bam);var _super2 = _createSuper(FakeBam);


  function FakeBam(remoteFile,
  remoteIndexFile,
  indexChunks) {var _this2;_classCallCheck(this, FakeBam);
    _this2 = _super2.call(this, remoteFile, remoteIndexFile, indexChunks);
    _this2.deferred = _q["default"].defer();return _this2;
  }_createClass(FakeBam, [{ key: "getFeaturesInRange", value:

    function getFeaturesInRange() {
      return this.deferred.promise;
    } }, { key: "release", value:

    function release(alignments) {
      this.deferred.resolve(alignments);
    } }]);return FakeBam;}(_bam["default"]);



describe('PileupTrack', function () {
  var testDiv = document.getElementById('testdiv');
  if (!testDiv) throw new Error("Failed to match: testdiv");

  beforeEach(function () {
    // A fixed width container results in predictable x-positions for mismatches.
    testDiv.style.width = '800px';
    _dataCanvas["default"].RecordingContext.recordAll();
  });

  afterEach(function () {
    _dataCanvas["default"].RecordingContext.reset();
    // avoid pollution between tests.
    testDiv.innerHTML = '';
  });

  // Test data files
  var twoBitFile = new _MappedRemoteFile["default"](
  '/test-data/hg19.2bit.mapped',
  [[0, 16383], [691179834, 691183928], [694008946, 694011447]]),
  bamFile = new _RemoteFile["default"]('/test-data/synth3.normal.17.7500000-7515000.bam'),
  bamIndexFile = new _RemoteFile["default"]('/test-data/synth3.normal.17.7500000-7515000.bam.bai');

  // It simplifies the tests to have these variables available synchronously.
  var reference = '',
  alignments = [];

  before(function () {
    var twoBit = new _TwoBit2["default"](twoBitFile),
    bam = new _bam["default"](bamFile, bamIndexFile);
    return twoBit.getFeaturesInRange('chr17', 7500000, 7510000).then(function (seq) {
      reference = seq;
      return bam.getFeaturesInRange(new _ContigInterval["default"]('chr17', 7500734, 7500795));
    }).then(function (result) {
      (0, _chai.expect)(result).to.have.length.above(0);
      alignments = result;
    });
  });

  function testSetup() {
    // The fake sources allow precise control over when they give up their data.
    var fakeTwoBit = new FakeTwoBit(twoBitFile),
    fakeBam = new FakeBam(bamFile, bamIndexFile),
    referenceSource = _TwoBitDataSource["default"].createFromTwoBitFile(fakeTwoBit),
    bamSource = _BamDataSource["default"].createFromBamFile(fakeBam);

    var p = _pileup["default"].create(testDiv, {
      range: { contig: 'chr17', start: 7500734, stop: 7500795 },
      tracks: [
      {
        data: referenceSource,
        viz: _pileup["default"].viz.genome(),
        isReference: true },

      {
        data: bamSource,
        viz: _pileup["default"].viz.pileup() }] });




    return { p: p, fakeTwoBit: fakeTwoBit, fakeBam: fakeBam };
  }var

  drawnObjectsWith = _dataCanvas["default"].RecordingContext.drawnObjectsWith;

  var hasReference = function hasReference() {
    // The reference initially shows "unknown" base pairs, so we have to
    // check for a specific known one to ensure that it's really loaded.
    return testDiv.querySelector('.reference canvas') !== null &&
    testDiv.querySelector('.reference canvas') !== undefined &&
    drawnObjectsWith(testDiv, '.reference', function (x) {return x.letter;}).length > 0;
  },
  hasAlignments = function hasAlignments() {
    return testDiv.querySelector('.pileup canvas') !== null &&
    testDiv.querySelector('.pileup canvas') !== undefined &&
    drawnObjectsWith(testDiv, '.pileup', function (x) {return x.span;}).length > 0;
  },
  hasPileupSelector = function hasPileupSelector() {
    return testDiv.querySelector('.pileup canvas') !== null &&
    testDiv.querySelector('.pileup canvas') !== undefined;
  },

  // Helpers for working with DataCanvas
  mismatchesAtPos = function mismatchesAtPos(pos) {return drawnObjectsWith(testDiv, '.pileup', function (x) {return x.basePair && x.pos == pos;});},

  // This checks that there are 22 C/T SNVs at chr17:7,500,765
  // XXX: IGV only shows 20
  assertHasColumnOfTs = function assertHasColumnOfTs() {
    var ref = drawnObjectsWith(testDiv, '.reference', function (x) {return x.pos == 7500765 - 1;});
    (0, _chai.expect)(ref).to.have.length(1);
    (0, _chai.expect)(ref[0].letter).to.equal('C');

    var mismatches = mismatchesAtPos(7500765 - 1);
    (0, _chai.expect)(mismatches).to.have.length(22);
    _underscore["default"].each(mismatches, function (mm) {
      (0, _chai.expect)(mm.basePair).to.equal('T');
    });
    // Make sure there are no variants in the previous column, just the reference.
    (0, _chai.expect)(mismatchesAtPos(7500764 - 1).length).to.equal(0);
  };

  it('should indicate mismatches when the reference loads first', function () {var _testSetup =
    testSetup(),p = _testSetup.p,fakeTwoBit = _testSetup.fakeTwoBit,fakeBam = _testSetup.fakeBam;

    // Release the reference first.
    fakeTwoBit.release(reference);

    // Wait for the track to render, then release the alignments.
    return (0, _async.waitFor)(hasReference, 2000).then(function () {
      fakeBam.release(alignments);
      return (0, _async.waitFor)(hasAlignments, 2000);
    }).then(function () {
      // Some number of mismatches are expected, but it should be dramatically
      // lower than the number of total base pairs in alignments.
      var mismatches = drawnObjectsWith(testDiv, '.pileup', function (x) {return x.basePair;});
      (0, _chai.expect)(mismatches).to.have.length.below(60);
      assertHasColumnOfTs();
      p.destroy();
    });
  });

  // Same as the previous test, but with the loads reversed.
  it('should indicate mismatches when the alignments load first', function () {var _testSetup2 =
    testSetup(),p = _testSetup2.p,fakeTwoBit = _testSetup2.fakeTwoBit,fakeBam = _testSetup2.fakeBam;

    // Release the alignments first.
    fakeBam.release(alignments);

    // Wait for the track to render, then release the reference.
    return (0, _async.waitFor)(hasAlignments, 2000).then(function () {
      fakeTwoBit.release(reference);
      return (0, _async.waitFor)(hasReference, 2000);
    }).then(function () {
      var mismatches = drawnObjectsWith(testDiv, '.pileup', function (x) {return x.basePair;});
      (0, _chai.expect)(mismatches).to.have.length.below(60);
      assertHasColumnOfTs();
      p.destroy();
    });
  });

  it('should hide alignments', function () {

    // The fake sources allow precise control over when they give up their data.
    var fakeTwoBit = new FakeTwoBit(twoBitFile),
    referenceSource = _TwoBitDataSource["default"].createFromTwoBitFile(fakeTwoBit);


    var p = _pileup["default"].create(testDiv, {
      range: { contig: 'chr17', start: 7500734, stop: 7500796 },
      tracks: [
      {
        data: referenceSource,
        viz: _pileup["default"].viz.genome(),
        isReference: true },

      {
        data: _pileup["default"].formats.bam({
          url: '/test-data/synth3.normal.17.7500000-7515000.bam',
          indexUrl: '/test-data/synth3.normal.17.7500000-7515000.bam.bai' }),

        viz: _pileup["default"].viz.pileup({
          hideAlignments: true }) }] });





    return (0, _async.waitFor)(hasPileupSelector, 2000).then(function () {
      var alignments = drawnObjectsWith(testDiv, '.pileup', function (x) {return x.span;});
      (0, _chai.expect)(alignments.length).to.equal(0);
      p.destroy();
    });
  });

  it('should sort reads', function () {

    // The fake sources allow precise control over when they give up their data.
    var fakeTwoBit = new FakeTwoBit(twoBitFile),
    referenceSource = _TwoBitDataSource["default"].createFromTwoBitFile(fakeTwoBit);


    var p = _pileup["default"].create(testDiv, {
      range: { contig: 'chr17', start: 7500734, stop: 7500796 },
      tracks: [
      {
        data: referenceSource,
        viz: _pileup["default"].viz.genome(),
        isReference: true },

      {
        data: _pileup["default"].formats.bam({
          url: '/test-data/synth3.normal.17.7500000-7515000.bam',
          indexUrl: '/test-data/synth3.normal.17.7500000-7515000.bam.bai' }),

        viz: _pileup["default"].viz.pileup({
          viewAsPairs: false }) }] });





    return (0, _async.waitFor)(hasAlignments, 2000).then(function () {
      var alignments = drawnObjectsWith(testDiv, '.pileup', function (x) {return x.span;});
      var center = (7500796 + 7500734) / 2;
      (0, _chai.expect)(Math.floor(center)).to.equal(center); // no rounding issues

      var rowsAndSpans = alignments.map(function (x) {return [x.row, x.span.interval];});
      var centerRows = _underscore["default"].uniq(
      rowsAndSpans.filter(function (rowIv) {return rowIv[1].contains(center);}).
      map(function (rowIv) {return rowIv[0];}));

      // The rows with alignments overlapping the center should be the first
      // ones, e.g. 0, 1, 2, 3. Any gaps will make this comparison fail.
      (0, _chai.expect)(_underscore["default"].max(centerRows)).to.equal(centerRows.length - 1);

      p.destroy();
    });
  });
});