


'use strict';

var _chai = require("chai");

var _pileup = _interopRequireDefault(require("../../main/pileup"));
var _dataCanvas = _interopRequireDefault(require("data-canvas"));
var _async = require("../async");
var _TwoBit = _interopRequireDefault(require("../../main/data/TwoBit"));
var _TwoBitDataSource = _interopRequireDefault(require("../../main/sources/TwoBitDataSource"));
var _MappedRemoteFile = _interopRequireDefault(require("../MappedRemoteFile"));
var _FakeTwoBit = require("../FakeTwoBit");

var _testUtils = _interopRequireDefault(require("react-dom/test-utils"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}

describe('VariantTrack', function () {
  var testDiv = document.getElementById('testdiv');
  if (!testDiv) throw new Error("Failed to match: testdiv");
  var reference = '';

  // Test data files
  var twoBitFile = new _MappedRemoteFile["default"](
  '/test-data/hg19.2bit.mapped',
  [[0, 16383], [691179834, 691183928], [694008946, 694011447]]);

  before(function () {
    var twoBit = new _TwoBit["default"](twoBitFile);
    return twoBit.getFeaturesInRange('17', 7500000, 7510000).then(function (seq) {
      reference = seq;
    });
  });

  beforeEach(function () {
    testDiv.style.width = '700px';
    _dataCanvas["default"].RecordingContext.recordAll();
  });

  afterEach(function () {
    _dataCanvas["default"].RecordingContext.reset();
    // avoid pollution between tests.
    testDiv.innerHTML = '';
  });var
  drawnObjects = _dataCanvas["default"].RecordingContext.drawnObjects;

  function ready() {
    return testDiv.getElementsByTagName('canvas').length > 0 &&
    drawnObjects(testDiv, '.variants').length > 0;
  }

  it('should render variants', function () {

    var fakeTwoBit = new _FakeTwoBit.FakeTwoBit(twoBitFile),
    referenceSource = _TwoBitDataSource["default"].createFromTwoBitFile(fakeTwoBit);

    // Release the reference first.
    fakeTwoBit.release(reference);

    var variantClickedData = null;
    var variantClicked = function variantClicked(data) {
      variantClickedData = data;
    };
    var p = _pileup["default"].create(testDiv, {
      range: { contig: '17', start: 9386380, stop: 9537390 },
      tracks: [
      {
        viz: _pileup["default"].viz.genome(),
        data: referenceSource,
        isReference: true },

      {
        data: _pileup["default"].formats.vcf({
          url: '/test-data/test.vcf' }),

        viz: _pileup["default"].viz.variants(),
        options: { onVariantClicked: variantClicked } }] });




    return (0, _async.waitFor)(ready, 2000).
    then(function () {
      var variants = drawnObjects(testDiv, '.variants');
      (0, _chai.expect)(variants.length).to.be.equal(1);
      var canvasList = testDiv.getElementsByTagName('canvas');
      var canvas = canvasList[1];
      (0, _chai.expect)(variantClickedData).to.be["null"];

      //check clicking on variant
      _testUtils["default"].Simulate.click(canvas, { nativeEvent: { offsetX: 0, offsetY: 17 } });

      (0, _chai.expect)(variantClickedData).to.not.be["null"];
      p.destroy();
    });
  });

});